%{
/*
 * $Id: fp_yacc.y,v 2.5 89/11/11 19:17:00 tynor Exp $
 *----------------------------------------------------------------------------
 *	FPLAN - Flight Planner
 *	Steve Tynor
 *	tynor@prism.gatech.edu
 *
 *	This program is in the public domain. Permission to copy,
 * distribute, modify this program is hearby given as long as this header
 * remains. If you redistribute this program after modifying it, please
 * document your changes so that I do not take the blame (or credit) for
 * those changes.  If you fix bugs or add features, please send me a
 * patch so that I can keep the 'official' version up-to-date.
 *
 *	Bug reports are welcome and I'll make an attempt to fix those
 * that are reported.
 *
 *	USE AT YOUR OWN RISK! I assume no responsibility for any
 * errors in this program, its database or documentation. I will make an
 * effort to fix bugs, but if you crash and burn because, for example,
 * fuel estimates in this program were inaccurate, it's your own fault
 * for trusting somebody else's code! Remember, as PIC, it's _your_
 * responsibility to do complete preflight planning. Use this program as
 * a flight planning aid, but verify its results before using them.
 *----------------------------------------------------------------------------
 */

#include "config.h"
#include "wp_info.h"
#include "proto.h"

static char rcsid[] = "$Id: fp_yacc.y,v 2.5 89/11/11 19:17:00 tynor Exp $";

typedef union {
   int			ival;
   double		dval;
   char			*sval;
   WAYPOINT_KIND	kval;
} YYSTYPE;

%}

%type <kval> waypoint_kind
   /*
    * %type <dval> radial
    */
%type <dval> number
%type <dval> latitude
%type <dval> longitude
%type <dval> heading
%type <dval> distance 
%type <dval> speed
%type <sval> optional_string
%type <sval> desig

%token TOK_SEMICOLON
%token TOK_ASSIGN
%token TOK_VIA
%token TOK_ALT
%token TOK_FROM
%token TOK_TO
%token TOK_MI
%token TOK_NM
%token TOK_MPH
%token TOK_KTS
%token TOK_COMMA
%token TOK_REAL
%token TOK_INTEGER
%token TOK_NAV
%token TOK_WIND
%token TOK_TAS
%token TOK_LPAREN
%token TOK_RPAREN
%token TOK_FUEL_AMT
%token TOK_FUEL_RATE
%token TOK_FUEL_USED
%token TOK_STRING
%token TOK_SYMBOL
%token TOK_ATSIGN

%start fplan

%%

fplan		: directives
		;

directives 	: /* empty */
		| directives directive
		;

directive	: named_waypoint	TOK_SEMICOLON
   		| inc_waypoint		TOK_SEMICOLON
   		/* | int_waypoint	TOK_SEMICOLON */
   		| lat_waypoint		TOK_SEMICOLON
		| nav_fix		TOK_SEMICOLON
		| true_airspeed		TOK_SEMICOLON
		| wind			TOK_SEMICOLON
		| fuel_amt		TOK_SEMICOLON
		| fuel_rate		TOK_SEMICOLON
		| fuel_used		TOK_SEMICOLON
   		| flight_altitude	TOK_SEMICOLON
		;

named_waypoint	: waypoint_kind desig 
			{add_named_waypoint ($1, $2);}

inc_waypoint 	: waypoint_kind distance 
   		  optional_string optional_string optional_string
			{add_inc_waypoint ($2, $3, $4, $5);}
		;

/*
 *int_waypoint	: waypoint_kind desig TOK_COMMA radial TOK_COMMA desig
 *		  TOK_COMMA radial optional_string
 *			{add_int_waypoint ($1, $2, $4, $6, $8, $9);}
 *		;
 */

lat_waypoint	: waypoint_kind latitude TOK_COMMA longitude 
   		  optional_string optional_string optional_string
		  {add_lat_waypoint ($1, $2, $4, $5, $6, $7);}
		;

waypoint_kind	: TOK_FROM {$$ = WP_FROM;}
		| TOK_VIA  {$$ = WP_VIA;}
		| TOK_TO   {$$ = WP_TO;}
		;

nav_fix		: TOK_NAV TOK_LPAREN number TOK_RPAREN  desig
		  {set_xfix ((int) $3, $5);}
		;

true_airspeed	: TOK_TAS  speed {set_tas ($2);}
		;

wind		: TOK_WIND  heading TOK_ATSIGN speed
		  {set_wind ($2, $4);}
		;

fuel_amt	: TOK_FUEL_AMT  number {set_fuel_amt ($2);}
		;

fuel_rate	: TOK_FUEL_RATE  number {set_fuel_rate ($2);}
		;

fuel_used	: TOK_FUEL_USED  number {set_extra_fuel_burn ($2);}
		;

flight_altitude	: TOK_ALT number {set_altitude ($2);}
		;

latitude	: TOK_REAL	{$$ = degrees_mins_2_decimal (yydval);}
		;

longitude	: TOK_REAL	{$$ = degrees_mins_2_decimal (yydval);}
		;

optional_string	: /* empty */	        {$$ = (char*) 0;}
		| TOK_COMMA TOK_STRING	{$$ = strdup (yytext);}
		;

desig		: TOK_SYMBOL    {$$ = strdup (yytext);}
		;

number		: TOK_REAL	{$$ = yydval;}
		| TOK_INTEGER	{$$ = yydval;}
		;

distance	: number TOK_MI {$$ = $1 / MI_PER_NM;}
   		| number 	{$$ = $1;}
   		| number TOK_NM {$$ = $1;}
		;

speed		: number TOK_MPH {$$ = $1 / MI_PER_NM;}
   		| number 	 {$$ = $1;}
   		| number TOK_KTS {$$ = $1;}
		;

/*
 *radial       	: heading
 *		;
 */

heading		: number	{$$ = $1;}
		;
