#ifndef _MAILCHECK_H_
#define _MAILCHECK_H_
/* $Id: mailcheck.h,v 1.3 1999/11/28 18:03:05 angus Exp $
 *
 * Interfaces for mailcheck program
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <assert.h>

/* Types */

typedef enum {false = 0, true = 1} bool ;

enum {RelaxFor = 1,		/* Optional '@' in Received: "for" */
      RelaxSemi = 2,		/* Semi-colons optional in Received: */
      RelaxTo = 4,		/* Optional '@' in mailbox */
      RelaxReturnPath = 8,	/* Optional '@' in Return-Path: */
      RelaxMailbox = 16,	/* Mailbox address may be in <> */
      RelaxMicrosoft = 32,	/* with Microsoft MAPI accepted */
      RelaxId = 64,		/* Dot accepted in Received: id */
      RelaxReferences = 128,	/* Anything for References and In-Reply-To */
      RelaxFrom = 256		/* Ignore From without colon */
} ;

extern int debug ;

/* Functions from mailcheck.c */
void error(char *format, ...) ;

/* Functions from block.c */
typedef unsigned int uint ; /* just an abbreviation */
void add_domain(char *domain, bool block) ;
bool block_domain(char *domain);
void add_user(char *user, bool block) ;
bool block_user(char *user) ;
void add_address(uint a, uint b, uint c, uint d, bool block) ;
bool block_address(uint a, uint b, uint c, uint d) ;
bool iterate_address(void **iterator, uint *aa, uint *bb, uint *cc, uint *dd) ;

/* Functions from readline.c */
char *readline(FILE *infile, long *lenptr, bool fold) ;
void flushlines(FILE *outfile) ;

/* Functions from rfc822.c */
bool keyword(char **lineptr, char *key) ;
bool received(char *line) ;
bool messageid(char *line) ;
bool returnpath(char *line) ;
bool dateheader(char *line) ;
bool mailbox(char *line) ;
bool mailboxes(char *line) ;
bool addresses(char *line) ;
bool references(char *line) ;

extern char *errorpos ; /* Pointer to where error occurred */
extern char *errorstr ; /* Reason for error */
extern int relax ;      /* How lax are we? */
extern bool rfc821 ;    /* Allow RFC821 lexemes */

/*
$Log: mailcheck.h,v $
Revision 1.3  1999/11/28 18:03:05  angus
Add Unix name resolution, blocked address update, etc.

Revision 1.2  1999/11/24 05:17:10  angus
Avoid blocking own domain

Revision 1.1.1.1  1999/11/24 04:18:51  angus
Initial import

*/

#endif /* _MAILCHECK_H_ */
