/* $Id: readline.c,v 1.1.1.1 1999/11/24 04:18:51 angus Exp $
 *
 * Read line interface for mailcheck program. Reads a line at a time from an
 * input file, storing original lines in buffer on each readline() call until
 * a flushlines() call is made. A folded canonical version is created and
 * stored in a transient buffer if the fold parameter is true. readline()
 * returns a pointer to a NULL terminated string with either the original
 * data or the folded version, including the terminating CR/LF. The return
 * value is NULL when the end of the header section is read, and is then NULL
 * again when EOF is reached.
 */

#include "mailcheck.h"
#include <memory.h>

#ifdef NO_MEMMOVE
void *memmove(void *vdest, void *vsrc, size_t count)
{
  char *dest = vdest, *src = vsrc ;

  if ( dest <= src ) {
    switch ( ((long)src & 3) ^ ((long)dest & 3) ) {
    case 0:			/* Same alignment */
      {
	register unsigned long *fromptr, *toptr ;
	register unsigned long n ;

	/* Do bytes until 32-bit aligned */
	switch ( (long)src & 3 ) {
	case 1:
	  *dest++ = *src++ ; count-- ;
	case 2:
	  *dest++ = *src++ ; count-- ;
	case 3:
	  *dest++ = *src++ ; count-- ;
	}

	fromptr = (unsigned long *)src ;
	toptr = (unsigned long *)dest ;
	n = count >> 2 ;

	while ( n >= 8 ) {
	  toptr[ 0 ] = fromptr[ 0 ] ;
	  toptr[ 1 ] = fromptr[ 1 ] ;
	  toptr[ 2 ] = fromptr[ 2 ] ;
	  toptr[ 3 ] = fromptr[ 3 ] ;
	  toptr[ 4 ] = fromptr[ 4 ] ;
	  toptr[ 5 ] = fromptr[ 5 ] ;
	  toptr[ 6 ] = fromptr[ 6 ] ;
	  toptr[ 7 ] = fromptr[ 7 ] ;
	  toptr += 8 ;
	  fromptr += 8 ;
	  n -= 8 ;
	}

	while ( n ) {
	  toptr[ 0 ] = fromptr[ 0 ] ;
	  toptr += 1 ;
	  fromptr += 1 ;
	  n -= 1 ;
	}
  
	src = (char *)fromptr ;
	dest = (char *)toptr ;

	switch ( count & 3 ) {
	case 3:
	  *dest++ = *src++ ;
	case 2:
	  *dest++ = *src++ ;
	case 1:
	  *dest++ = *src++ ;
	}
      }
      break ;
    case 2:			/* 16-bit transfers possible */
      {
	register unsigned short *fromptr, *toptr ;
	register unsigned long n ;

	/* Do bytes until 16-bit aligned */
	if ( (long)src & 1 ) {
	  *dest++ = *src++ ;
	  count-- ;
	}

	fromptr = (unsigned short *)src ;
	toptr = (unsigned short *)dest ;
	n = count >> 1 ;

	while ( n >= 16 ) {
	  toptr[ 0 ] = fromptr[ 0 ] ;
	  toptr[ 1 ] = fromptr[ 1 ] ;
	  toptr[ 2 ] = fromptr[ 2 ] ;
	  toptr[ 3 ] = fromptr[ 3 ] ;
	  toptr[ 4 ] = fromptr[ 4 ] ;
	  toptr[ 5 ] = fromptr[ 5 ] ;
	  toptr[ 6 ] = fromptr[ 6 ] ;
	  toptr[ 7 ] = fromptr[ 7 ] ;
	  toptr[ 8 ] = fromptr[ 8 ] ;
	  toptr[ 9 ] = fromptr[ 9 ] ;
	  toptr[ 10 ] = fromptr[ 10 ] ;
	  toptr[ 11 ] = fromptr[ 11 ] ;
	  toptr[ 12 ] = fromptr[ 12 ] ;
	  toptr[ 13 ] = fromptr[ 13 ] ;
	  toptr[ 14 ] = fromptr[ 14 ] ;
	  toptr[ 15 ] = fromptr[ 15 ] ;
	  toptr += 16 ;
	  fromptr += 16 ;
	  n -= 16 ;
	}

	while ( n ) {
	  toptr[ 0 ] = fromptr[ 0 ] ;
	  toptr += 1 ;
	  fromptr += 1 ;
	  n -= 1 ;
	}

	if ( count & 1 ) {
	  src = (char *)fromptr ;
	  dest = (char *)toptr ;
	  dest[ 0 ] = src[ 0 ] ;
	}
      }
      break ;
    case 1:			/* Badly out of phase, use byte transfer */
    case 3:
      {
	register unsigned char *fromptr, *toptr ;
	register unsigned long n = count ;

	fromptr = (unsigned char *)src ;
	toptr = (unsigned char *)dest ;

	while ( n >= 32 ) {
	  toptr[ 0 ] = fromptr[ 0 ] ;
	  toptr[ 1 ] = fromptr[ 1 ] ;
	  toptr[ 2 ] = fromptr[ 2 ] ;
	  toptr[ 3 ] = fromptr[ 3 ] ;
	  toptr[ 4 ] = fromptr[ 4 ] ;
	  toptr[ 5 ] = fromptr[ 5 ] ;
	  toptr[ 6 ] = fromptr[ 6 ] ;
	  toptr[ 7 ] = fromptr[ 7 ] ;
	  toptr[ 8 ] = fromptr[ 8 ] ;
	  toptr[ 9 ] = fromptr[ 9 ] ;
	  toptr[ 10 ] = fromptr[ 10 ] ;
	  toptr[ 11 ] = fromptr[ 11 ] ;
	  toptr[ 12 ] = fromptr[ 12 ] ;
	  toptr[ 13 ] = fromptr[ 13 ] ;
	  toptr[ 14 ] = fromptr[ 14 ] ;
	  toptr[ 15 ] = fromptr[ 15 ] ;
	  toptr[ 16 ] = fromptr[ 16 ] ;
	  toptr[ 17 ] = fromptr[ 17 ] ;
	  toptr[ 18 ] = fromptr[ 18 ] ;
	  toptr[ 19 ] = fromptr[ 19 ] ;
	  toptr[ 20 ] = fromptr[ 20 ] ;
	  toptr[ 21 ] = fromptr[ 21 ] ;
	  toptr[ 22 ] = fromptr[ 22 ] ;
	  toptr[ 23 ] = fromptr[ 23 ] ;
	  toptr[ 24 ] = fromptr[ 24 ] ;
	  toptr[ 25 ] = fromptr[ 25 ] ;
	  toptr[ 26 ] = fromptr[ 26 ] ;
	  toptr[ 27 ] = fromptr[ 27 ] ;
	  toptr[ 28 ] = fromptr[ 28 ] ;
	  toptr[ 29 ] = fromptr[ 29 ] ;
	  toptr[ 30 ] = fromptr[ 30 ] ;
	  toptr[ 31 ] = fromptr[ 31 ] ;
	  toptr += 32 ;
	  fromptr += 32 ;
	  n -= 32 ;
	}

	while ( n ) {
	  toptr[ 0 ] = fromptr[ 0 ] ;
	  toptr += 1 ;
	  fromptr += 1 ;
	  n -= 1 ;
	}
      }
      break ;
    }
  } else { /* dest > src */
    src += count ;
    dest += count ;

    switch ( ((long)src & 3) ^ ((long)dest & 3) ) {
    case 0:			/* Same alignment */
      {
	register unsigned long *fromptr, *toptr ;
	register unsigned long n ;

	/* Do bytes until 32-bit aligned */
	switch ( (long)src & 3 ) {
	case 3:
	  *--dest = *--src ; count-- ;
	case 2:
	  *--dest = *--src ; count-- ;
	case 1:
	  *--dest = *--src ; count-- ;
	}

	fromptr = (unsigned long *)src ;
	toptr = (unsigned long *)dest ;
	n = count >> 2 ;

	while ( n >= 8 ) {
	  toptr -= 8 ;
	  fromptr -= 8 ;
	  toptr[ 7 ] = fromptr[ 7 ] ;
	  toptr[ 6 ] = fromptr[ 6 ] ;
	  toptr[ 5 ] = fromptr[ 5 ] ;
	  toptr[ 4 ] = fromptr[ 4 ] ;
	  toptr[ 3 ] = fromptr[ 3 ] ;
	  toptr[ 2 ] = fromptr[ 2 ] ;
	  toptr[ 1 ] = fromptr[ 1 ] ;
	  toptr[ 0 ] = fromptr[ 0 ] ;
	  n -= 8 ;
	}

	while ( n ) {
	  *--toptr = *--fromptr ;
	  n -= 1 ;
	}
  
	src = (char *)fromptr ;
	dest = (char *)toptr ;

	switch ( count & 3 ) {
	case 3:
	  *--dest = *--src ;
	case 2:
	  *--dest = *--src ;
	case 1:
	  *--dest = *--src ;
	}
      }
      break ;
    case 2:			/* 16-bit transfers possible */
      {
	register unsigned short *fromptr, *toptr ;
	register unsigned long n ;

	/* Do bytes until 16-bit aligned */
	if ( (long)src & 1 ) {
	  *--dest = *--src ;
	  count-- ;
	}

	fromptr = (unsigned short *)src ;
	toptr = (unsigned short *)dest ;
	n = count >> 1 ;

	while ( n >= 16 ) {
	  toptr -= 16 ;
	  fromptr -= 16 ;
	  toptr[ 15 ] = fromptr[ 15 ] ;
	  toptr[ 14 ] = fromptr[ 14 ] ;
	  toptr[ 13 ] = fromptr[ 13 ] ;
	  toptr[ 12 ] = fromptr[ 12 ] ;
	  toptr[ 11 ] = fromptr[ 11 ] ;
	  toptr[ 10 ] = fromptr[ 10 ] ;
	  toptr[ 9 ] = fromptr[ 9 ] ;
	  toptr[ 8 ] = fromptr[ 8 ] ;
	  toptr[ 7 ] = fromptr[ 7 ] ;
	  toptr[ 6 ] = fromptr[ 6 ] ;
	  toptr[ 5 ] = fromptr[ 5 ] ;
	  toptr[ 4 ] = fromptr[ 4 ] ;
	  toptr[ 3 ] = fromptr[ 3 ] ;
	  toptr[ 2 ] = fromptr[ 2 ] ;
	  toptr[ 1 ] = fromptr[ 1 ] ;
	  toptr[ 0 ] = fromptr[ 0 ] ;
	  n -= 16 ;
	}

	while ( n ) {
	  *--toptr = *--fromptr ;
	  n -= 1 ;
	}

	if ( count & 1 ) {
	  src = (char *)fromptr ;
	  dest = (char *)toptr ;
	  *--dest = *--src ;
	}
      }
      break ;
    case 1:			/* Badly out of phase, use byte transfer */
    case 3:
      {
	register unsigned char *fromptr, *toptr ;
	register unsigned long n = count ;

	fromptr = (unsigned char *)src ;
	toptr = (unsigned char *)dest ;

	while ( n >= 32 ) {
	  toptr -= 32 ;
	  fromptr -= 32 ;
	  toptr[ 31 ] = fromptr[ 31 ] ;
	  toptr[ 30 ] = fromptr[ 30 ] ;
	  toptr[ 29 ] = fromptr[ 29 ] ;
	  toptr[ 28 ] = fromptr[ 28 ] ;
	  toptr[ 27 ] = fromptr[ 27 ] ;
	  toptr[ 26 ] = fromptr[ 26 ] ;
	  toptr[ 25 ] = fromptr[ 25 ] ;
	  toptr[ 24 ] = fromptr[ 24 ] ;
	  toptr[ 23 ] = fromptr[ 23 ] ;
	  toptr[ 22 ] = fromptr[ 22 ] ;
	  toptr[ 21 ] = fromptr[ 21 ] ;
	  toptr[ 20 ] = fromptr[ 20 ] ;
	  toptr[ 19 ] = fromptr[ 19 ] ;
	  toptr[ 18 ] = fromptr[ 18 ] ;
	  toptr[ 17 ] = fromptr[ 17 ] ;
	  toptr[ 16 ] = fromptr[ 16 ] ;
	  toptr[ 15 ] = fromptr[ 15 ] ;
	  toptr[ 14 ] = fromptr[ 14 ] ;
	  toptr[ 13 ] = fromptr[ 13 ] ;
	  toptr[ 12 ] = fromptr[ 12 ] ;
	  toptr[ 11 ] = fromptr[ 11 ] ;
	  toptr[ 10 ] = fromptr[ 10 ] ;
	  toptr[ 9 ] = fromptr[ 9 ] ;
	  toptr[ 8 ] = fromptr[ 8 ] ;
	  toptr[ 7 ] = fromptr[ 7 ] ;
	  toptr[ 6 ] = fromptr[ 6 ] ;
	  toptr[ 5 ] = fromptr[ 5 ] ;
	  toptr[ 4 ] = fromptr[ 4 ] ;
	  toptr[ 3 ] = fromptr[ 3 ] ;
	  toptr[ 2 ] = fromptr[ 2 ] ;
	  toptr[ 1 ] = fromptr[ 1 ] ;
	  toptr[ 0 ] = fromptr[ 0 ] ;
	  n -= 32 ;
	}

	while ( n ) {
	  *--toptr = *--fromptr ;
	  n -= 1 ;
	}
      }
      break ;
    }
  }

  return vdest ;
}
#endif

static char *buffer = NULL, nextch = '\0' ;
static long bufsize = 0, bufread = 0, bufseen = 0 ;

char *readline(FILE *infile, long *lenptr, bool fold)
{
  long spare = bufsize - bufread, start = bufseen ;

  /* Restore terminator character */
  if ( bufread > bufseen )
    buffer[bufseen] = nextch ;

  for (;;) {
    /* Retain one extra character, partly for testing folding, and partly for
       terminator */
    if ( bufseen + 1 >= bufread ) { /* Read some more */
      long rbytes ;

      if ( spare < 100 ) { /* Re-allocate before reading anything */
        bufsize = bufsize == 0 ? 4096 : bufsize * 2 ;

        if ( (buffer = realloc(buffer, bufsize)) == NULL )
          error("out of memory reading message") ;
        spare = bufsize - bufread ;
      }

      if ( (rbytes = fread(buffer, sizeof(char), spare, infile)) < 0 )
        error("Problem reading line from input file") ;

      if ( rbytes == 0 ) {
        bufseen = bufread ; /* There is no more. Return last line (if any) */
        break ;
      }

      bufread += rbytes ;
    }

    if ( buffer[bufseen++] == '\n' ) { /* Got LF, now test for CRLF */
      if ( bufseen - start >= 2 && buffer[bufseen - 2] == '\r' ) {
        if ( !fold ) /* Can line be folded? */
          break ;

        if ( bufseen - start == 2 ) /* Empty line. Cannot be folded. */
          return NULL ;

        if ( buffer[bufseen] != ' ' && buffer[bufseen] != '\t' ) /* Fold? */
          break ;
      }
    }
  }

  /* If we got here, we have one or more lines between buffer[start] and
     buffer[bufseen]. If there are multiple lines, they may need unfolding
     and canonicalising, in which case we copy them to a temporary buffer,
     joining lines as we go, and removing spaces. */

  if ( bufseen <= start ) /* No line, must be at EOF */
    return NULL ;

  /* Terminate buffer properly, storing terminator character for retrieval */
  nextch = buffer[bufseen] ;
  buffer[bufseen] = '\0' ;

  if ( fold ) {
    static char *foldbuf = NULL ;
    static long foldsize = 0 ;

    char *dest, *src ;
    bool space = false ;

    if ( bufseen - start >= foldsize ) {
      foldsize = bufseen - start + 1 ; /* New maximum size */
      if ( (foldbuf = realloc(foldbuf, foldsize)) == NULL )
        error("Out of memory allocating folding buffer") ;
    }

    for ( src = buffer + start, dest = foldbuf ; *src ; ++src ) {
      switch ( *src ) {
      case '\r': case '\n': case ' ': case '\t': /* Space character */
        space = true ;
        break ;
      default: /* Copy other chars */
        if ( space )
          *dest++ = ' ' ;
        *dest++ = *src ; /* FALLTHRU */
      case '\b': /* Remove backspace */
        space = false ;
        break ;
      }
    }
    *dest = '\0' ;

    *lenptr = dest - foldbuf ;

    return foldbuf ;
  }

  *lenptr = bufseen - start ;

  return buffer + start ;
}

void flushlines(FILE *outfile)
{
  if ( bufseen > 0 && outfile != NULL &&
       bufseen != fwrite(buffer, sizeof(char), bufseen, outfile) )
    error("Problem flushing lines read") ;

  if ( bufread > bufseen )
    memmove(buffer, buffer + bufseen, bufread - bufseen) ;

  bufread -= bufseen ;
  bufseen = 0 ;
}

/*
$Log: readline.c,v $
Revision 1.1.1.1  1999/11/24 04:18:51  angus
Initial import

*/
