/* artdump.c $Id: artdump.c,v 1.3 2000/01/14 21:22:34 angus Exp $
 *
 * Copyright (C) Angus J. C. Duggan, 1997-1998
 *
 * Freely redistributably software. See file LICENSE for details.
 *
 * Article dump utility for NNTP Buffer.
 */

#include "nntp.h"
#include "messages.h"

char *program ;
int debuglevel = 0x80000000 ;

char *confdir = NULL ;
char *spooldir = NULL ;
char *postdir = NULL ;
unsigned short inport, outport ;

long remember_max = 0 ;

BOOL is_service = FALSE ;

char *pattern = NULL ;

static void usage(void)
{
  fprintf(stderr, "Usage: %s [-dconfigdir] [-sspooldir] [pattern]\n", program) ;
  fprintf(stderr,
	  "Copyright (C) Angus J. C. Duggan, 1997-1998. Freely redistributable software\n"
	  "See file LICENSE for details.\n") ;
  ExitProcess(0) ;
}

int parse_args(int argc, char *argv[])
{
  int pathlen ;
  char path[MAX_PATH], *arg ;

  pathlen = GetModuleFileName(NULL, path, MAX_PATH) ;
  if ( pathlen ) {
    char *slash = strrchr(path, '\\') ;

    if ( slash ) {
      pathlen = slash - path ;
      *slash = '\0' ;
    }
  }

  for ( program = *argv++ ; --argc ; argv++ ) {
    if (argv[0][0] == '-') {
      char ch = argv[0][1] ;

      if ( strchr("dsD", ch) != NULL ) {
	if ( argv[0][2] != '\0' )
	  arg = &argv[0][2] ;
	else if ( --argc )
	  arg = *++argv ;
	else
	  return 0 ;
      } else if ( argv[0][2] != '\0' )
        return 0 ;

      switch (ch) {
      case 'd':	/* set config directory */
        confdir = arg ;
	break;
      case 's':	/* set spool directory */
        spooldir = arg ;
	break;
      case 'D':	/* debug level service */
        debuglevel |= strtol(arg, &arg, 0) ;
	if ( debuglevel == 0 )
	  return 0 ;
        break ;
      default:
	return 0 ;
      }
    } else if ( pattern == NULL )
      pattern = *argv ;
    else
      return 0 ;
  }

  if ( program == NULL )
    return 0 ;

  if ( pattern == NULL )
    pattern = "*" ;

  if ( !DefaultDirectories(path, pathlen) )
    return 0 ;

  return 1 ;
}

int DumpArticle(char *msgid, void *passthrough)
{
  Article *art ;

  if ( !Match(pattern, msgid) )
    return TRUE ;

  if ( !Lock(ARTICLE_MUTEX, INFINITE) )
    return FALSE ;

  printf("Message <%s>", msgid) ;

  if ( (art = ArticleDBGet(msgid)) != NULL ) {
    char *names = art->names, *scan = names ;

    switch (art->state) {
    case ARTICLE_BODY:
      printf("  (BODY)") ; break ;
    case ARTICLE_HEAD:
      printf("  (HEAD)") ; break ;
    case ARTICLE_EXPIRED:
      printf("  (EXPIRED)") ; break ;
    case ARTICLE_NEW:
      printf("  (NEW)") ; break ;
    default:
      printf("  (CORRUPT)") ; break ;
    }
    printf("  Size %d", art->size) ;
    printf("  Mark %d\n", art->mark) ;
    printf("  Downloaded %s", TimeAsString(&art->download)) ;
    printf("  Expires %s\n", TimeAsString(&art->expires)) ;
    while ( *names ) {
      printf("  %s\n", names) ;
      names += strlen(names) + 1 ;
    }
  } else
    printf("  Not found\n") ;
  
  Unlock(ARTICLE_MUTEX) ;

  return TRUE ;
}

void LogString(int type, char *str)
{
  fputs(str, stderr) ;
  fflush(stderr) ;
}

int main(int argc, char *argv[])
{
  if ( !parse_args(argc, argv) )
    usage() ;

  if ( !InitArticleDB() || !InitMessageDB() ) {
    fprintf(stderr, "Can't initialise article or message database") ;
    return 0 ;
  }
    
  ForAllArticles(DumpArticle, NULL) ;

  CloseArticleDB() ;
  CloseMessageDB() ;

  return 0 ;
}

/*
 * $Log: artdump.c,v $
 * Revision 1.3  2000/01/14 21:22:34  angus
 * More changes to logging. Use new streams 1.31 ioctl to specify debugging info
 * for client & server connections. Make fmtout more secure against buffer
 * overruns, and add a new format specifier for binary strings with visible
 * control characters. Change logging interface to add a varargs log, and use
 * that for stream debugging. Modify debug flags, add more status returns.
 *
 * Revision 1.2  2000/01/14 03:40:04  angus
 * Many changes; added named pipe logging, sending output to client debug
 * process for in-situ service debugging. Use this mechanism for queue status,
 * and enable after-the-fact debugging by saving 100 logged messages. Add
 * symbollic names for debug flags, and Mutex debugging.
 *
 * Revision 1.1.1.1  1999/10/25 07:49:57  angus
 * Initial CVS import
 *
 */

