/* common.c $Id: common.c,v 1.3 2000/01/14 21:22:34 angus Exp $
 *
 * Copyright (C) Angus J. C. Duggan, 1997-1998.
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Common parts for NNTP service for Windows NT.
 */

#include "nntp.h"
#include "messages.h"

#define MAX_FORMAT 10 /* Max format specification length */

char *fmtout(char *buffer, int bufsize, char *format, va_list args)
{
  int c ;
  char *here = buffer, *bufend = buffer + bufsize - 1 ;

  assert(format != NULL) ;

  while ( here < bufend && (c = (unsigned char)*format++) != 0 ) {
    if ( c == '%' ) {
      int done, index = 0, longform = 0 ;
      char fmtbuf[MAX_FORMAT] ;

      fmtbuf[index++] = c ;
      do {
	done = 1 ;
	fmtbuf[index++] = c = *format++ ;
	fmtbuf[index] = '\0' ;
	switch (c) {
	case '%':
	  *here++ = c ;
	case '\0':
	  break ;
	case 'e': case 'E': case 'f': case 'g': case 'G':
	  if ( here + 20 < bufend ) {
	    double d = va_arg(args, double) ;
	    sprintf(here, fmtbuf, d) ;
	    here += strlen(here) ;
	  }
	  break ;
	case 'c': case 'd': case 'i': case 'o':
	case 'p': case 'u': case 'x': case 'X':
          if ( here + 20 < bufend ) {
            if ( longform ) {
              long l = va_arg(args, long) ;
              sprintf(here, fmtbuf, l) ;
              here += strlen(here) ;
            } else {
              int i = va_arg(args, int) ;
              sprintf(here, fmtbuf, i) ;
              here += strlen(here) ;
            }
          }
	  break ;
	case 's':
	  {
	    char *s = va_arg(args, char *) ;

            if ( here + strlen(s) < bufend ) {
              sprintf(here, fmtbuf, s) ;
              here += strlen(here) ;
            }
	  }
	  break ;
	case 'S': /* string with control characters visible */
	  {
	    char *s = va_arg(args, char *) ;
            long len = va_arg(args, long) ;
            while ( len-- && here < bufend ) {
              char ch = *s++ ;
              switch ( ch ) {
              case '\r': strcpy(here, "<CR>") ; here += 4 ; break ;
              case '\n': strcpy(here, "<NL>\n") ; here += 5 ; break ;
              case '\t': strcpy(here, "<TAB>") ; here += 5 ; break ;
              default: *here++ = ch ; break ;
              }
            }
	  }
	  break ;
	case 't': /* FILETIME struct */
	  if ( here + 21 < bufend ) {
	    SYSTEMTIME systime ;
	    if ( FileTimeToSystemTime(va_arg(args, FILETIME *), &systime) ) {
	      static char *month[] = { "",
				       "Jan", "Feb", "Mar",
				       "Apr", "May", "Jun",
				       "Jul", "Aug", "Sep",
				       "Oct", "Nov", "Dec" } ;
	      sprintf(here, "%d %s %04d %02d:%02d:%02d", systime.wDay,
		      month[systime.wMonth],
		      systime.wYear < 1000 ? systime.wYear + 1900 : systime.wYear,
		      systime.wHour, systime.wMinute, systime.wSecond) ;
	      here += strlen(here) ;
	    }
	  }
	  break ;
	case 'l':
	  longform = 1 ;
	  /* FALLTHRU */
	default:
	  done = 0 ;
	}
      } while ( !done ) ;
    } else	/* Not %, write out character */
      *here++ = c ;
  }

  *here = '\0' ;

  return buffer ;
}

char *formatstr(char *buffer, int bufsize, char *format, ...)
{
  char *result ;
  va_list args ;

  va_start(args, format) ;
  result = fmtout(buffer, bufsize, format, args) ;
  va_end(args) ;

  return result ;
}

void debug(int level, char *format, ...)
{
  va_list args ;

  va_start(args, format) ;
  if ( (debuglevel & level) != 0 ) {
    char buffer[1024] ;
    LogString(LOG_DEBUG, fmtout(buffer, 1024, format, args)) ;
  }
  va_end(args) ;
}

/* Type is
   EVENTLOG_ERROR_TYPE	Error event
   EVENTLOG_WARNING_TYPE	Warning event
   EVENTLOG_INFORMATION_TYPE	Information event
   */
void message(WORD type, DWORD id, ...)
{
  va_list args ;

  va_start(args, id) ;
  if ( debuglevel == 0 && is_service ) {
    HANDLE hEventSource = RegisterEventSource(NULL, SERVICE_NAME);
    char *strings[20], *arg ;
    WORD inserts = 0 ;

    while ( (arg = va_arg(args, char*)) != NULL && inserts < 20 )
      strings[inserts++] = arg ;

    if ( hEventSource != NULL ) {
      ReportEvent(hEventSource,	/* handle of event source */
		  type,		/* event type */
		  0,		/* event category */
		  id,		/* event ID */
		  NULL,		/* current user's SID */
		  inserts,	/* number of strings */
		  0,		/* no bytes of raw data */
		  strings,	/* array of insert strings */
		  NULL) ;	/* raw data */
    
      (VOID)DeregisterEventSource(hEventSource) ;
    }
  } else {	/* Use FormatMessage to write out to stderr */
    char buffer[4000] ;

    if ( FormatMessage(FORMAT_MESSAGE_FROM_HMODULE,
		       GetModuleHandle(NULL),	/* Current module */
		       id,	/* From parameters */
		       LANG_NEUTRAL,	/* Default language */
		       buffer,	/* buffer address */
		       4000,	/* buffer size */
		       &args) != 0 ) {
      LogString(LOG_STATUS, buffer) ;
    } else {
      fprintf(stderr, "FormatMessage error %s\n", ErrorText(GetLastError())) ;
      fflush(stderr) ;
    }
  }
  va_end(args) ;
}

void Log(int type, char *format, ...)
{
  char buffer[1024] ;
  va_list args ;

  va_start(args, format) ;
  LogString(type, fmtout(buffer, 1024, format, args)) ;
  va_end(args) ;
}

/* Trawl over all files with names consisting solely of digits in directory */
int ForAllFiles(char *directory, char *extn,
		int (*fn)(char *filename, void *passthrough),
		void *passthrough)
{
  char pattern[MAX_PATH] ;
  WIN32_FIND_DATA fdata ;
  HANDLE it ;
  int dirlen = strlen(directory) ;
  int result = TRUE ;

  strcpy(pattern, directory) ;
  strcpy(pattern + dirlen, "\\*") ;
  strcpy(pattern + dirlen + 2, extn) ;

  if ( (it = FindFirstFile(pattern, &fdata)) != INVALID_HANDLE_VALUE ) {
    do {
      char *scan = fdata.cFileName ;

      while (isdigit(*scan)) scan++ ;	/* Skip digits in filename */

      debug(DEBUG_FILES, "Pattern=%s, file=%s\n", pattern, fdata.cFileName) ;

      if ( scan != fdata.cFileName && strcmp(scan, extn) == 0 ) {
	strcpy(pattern + dirlen + 1, fdata.cFileName) ;
	if ( !(*fn)(pattern, passthrough) ) {
	  result = FALSE ;
	  break ;
	}
      }
    } while (FindNextFile(it, &fdata)) ;
    if ( GetLastError() != ERROR_NO_MORE_FILES )
      result = FALSE ;
    FindClose(it) ;
  } else	/* FindFirstFile failed */
    result = FALSE ;

  return result ;
}

char *ErrorText(DWORD err)
{
  DWORD dwRet;
  char *string = NULL ;

  static char *longest = NULL ;

  dwRet = FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|
			FORMAT_MESSAGE_FROM_SYSTEM,
			NULL,
			err,
			LANG_NEUTRAL,
			(LPTSTR)&string,
			0,
			NULL);

  /* Save longest buffer to copy results into */
  if ( dwRet == 0 ) {      /* No error string; what to do? */
    static char message[64] ;

    sprintf(message, "error %d (%x)", err, err) ;
    return message ;
  } else if ( longest == NULL ) {
    longest = string ;
  } else if ( dwRet > strlen(longest) ) {
    LocalFree((HLOCAL)longest) ;
    longest = string ;
  } else {
    strcpy(longest, string) ;
    LocalFree((HLOCAL)string);
  }

  assert(longest != NULL) ;

  return longest ;
}

static HANDLE mutexes[N_MUTEXES] = { NULL } ;

#define MUTEX(name) "Mutex/" SERVICE_NAME "/" name
static char *mutex_names[N_MUTEXES] = {
  MUTEX("Mark"),
  MUTEX("Event"),
  MUTEX("Articles"),
  MUTEX("Messages"),
  MUTEX("Groups"),
  MUTEX("Server"),
  MUTEX("Expire"),
  MUTEX("Slurp"),
  MUTEX("Post"),
  MUTEX("Control")
} ;

LockResult Lock(LockID lid, DWORD timeout)
{
  HANDLE mutex ;

  if ( (mutex = mutexes[lid]) == NULL ) { /* try to create mutex */
    BOOL already = FALSE ;
    SECURITY_DESCRIPTOR sdesc ;
    SECURITY_ATTRIBUTES sattr = { sizeof(SECURITY_ATTRIBUTES), (LPVOID)&sdesc,
				  FALSE } ;

    debug(DEBUG_MUTEX, "Creating mutex %s\n", mutex_names[lid]) ;

    /* Set discretionary ACL to grant object access */
    if ( InitializeSecurityDescriptor(&sdesc, SECURITY_DESCRIPTOR_REVISION) ) {
      DWORD len = GetSidLengthRequired(1) ;
      PSID sid ;

      if ( (sid = (PSID)malloc(len)) != NULL ) {
	PACL acl ;

	len += sizeof(ACL) + sizeof(ACCESS_ALLOWED_ACE) - sizeof (DWORD) ;

	if ( (acl = (PACL)malloc(len)) != NULL ) {      
	  SID_IDENTIFIER_AUTHORITY siaWorld = SECURITY_WORLD_SID_AUTHORITY ;
	  if ( InitializeAcl(acl, len, ACL_REVISION) &&
	       InitializeSid(sid, &siaWorld, 1) ) {
	    *(GetSidSubAuthority(sid, 0)) = SECURITY_WORLD_RID ;

	    if ( AddAccessAllowedAce(acl, ACL_REVISION,
				     GENERIC_ALL|STANDARD_RIGHTS_ALL|SPECIFIC_RIGHTS_ALL,
				     sid) &&
		 SetSecurityDescriptorDacl(&sdesc, TRUE, acl, FALSE) ) {
	      mutex = CreateMutex(&sattr, FALSE, mutex_names[lid]) ;
	      already = (GetLastError() == ERROR_ALREADY_EXISTS) ;
	    }
	  }
	  free(acl) ;
	}
	free(sid) ;
      }
    }

    if ( mutex == NULL ) { /* Can't create mutex */
      message(EVENTLOG_ERROR_TYPE, MSG_CREATEMUTEX, mutex_names[lid],
	      ErrorText(GetLastError()), NULL) ;
      return LockFailed ;
    } else if ( already ) { /* Beaten to it by another thread */
      mutexes[lid] = mutex ;
    } else { /* we got there first, save mutex */
      mutexes[lid] = mutex ;
    }
  }

  debug(DEBUG_MUTEX, "Wait for mutex %s\n", mutex_names[lid]) ;
  switch (WaitForSingleObject(mutex, timeout)) {
  case WAIT_FAILED:
    message(EVENTLOG_ERROR_TYPE, MSG_ACQUIREMUTEX, mutex_names[lid],
	    ErrorText(GetLastError()), NULL) ;
    debug(DEBUG_MUTEX, "Failed to aquire mutex %s\n", mutex_names[lid]) ;
    return LockFailed ;
  case WAIT_TIMEOUT:
    debug(DEBUG_MUTEX, "Timeout on mutex %s\n", mutex_names[lid]) ;
    return LockTimeout ;
  }

  debug(DEBUG_MUTEX, "Granted mutex %s\n", mutex_names[lid]) ;

  return LockOK ;
}

void Unlock(LockID lid)
{
  HANDLE mutex = mutexes[lid] ;

  debug(DEBUG_MUTEX, "Releasing mutex %s\n", mutex_names[lid]) ;

  if ( mutex == NULL || !ReleaseMutex(mutex) ) {
    message(EVENTLOG_ERROR_TYPE, MSG_RELEASEMUTEX, mutex_names[lid],
	    ErrorText(GetLastError()), NULL) ;
  }
}


/* Return a new mark, guaranteed not equal to zero */
DWORD Mark(void)
{
  HKEY key ;
  DWORD type, disp, size = sizeof(DWORD) ;
  static DWORD mark = 0 ;

  if ( !Lock(MARK_MUTEX, INFINITE) )
    return mark ;

  if ( RegCreateKeyEx(HKEY_LOCAL_MACHINE, SERVICE_REGISTRY_KEY,
		      0, NULL, 0, KEY_WRITE|KEY_READ, NULL, &key,
		      &disp) == ERROR_SUCCESS ) {
    (void)RegQueryValueEx(key, "Mark", NULL, &type, (char *)&mark, &size) ;

    while (++mark == 0) ;	/* Get non-zero mark */

    if ( RegSetValueEx(key, "Mark", 0, REG_DWORD, (char *)&mark,
		       sizeof(DWORD)) != ERROR_SUCCESS ) {
      char buffer[20] ;
      sprintf(buffer, "%d", mark) ;
      message(EVENTLOG_WARNING_TYPE, MSG_REGSET, SERVICE_REGISTRY_KEY, "Mark",
	      buffer, ErrorText(GetLastError()), NULL) ;
    }

    if ( RegCloseKey(key) != ERROR_SUCCESS )
      message(EVENTLOG_ERROR_TYPE, MSG_REGCLOSE, ErrorText(GetLastError()),
	      NULL) ;
  } else {
    message(EVENTLOG_WARNING_TYPE, MSG_REGCREATE, SERVICE_REGISTRY_KEY,
	    ErrorText(GetLastError()), NULL) ;
    while (++mark == 0) ;	/* Get non-zero mark */
  }

  Unlock(MARK_MUTEX) ;

  return mark ;
}

int DefaultDirectories(char *path, int pathlen)
{
  /* Set config, spool and posting directories. Create dir and set registry
     key if installing service, delete dir if removing service */
  if ( confdir == NULL &&
       (confdir = GetRegString(SERVICE_REGISTRY_KEY, CONFIG_REGNAME)) == NULL ) {
    if ( (confdir = malloc(pathlen + strlen(CONFIG_DIRNAME) + 1)) == NULL ) {
      message(EVENTLOG_ERROR_TYPE, MSG_MEMORY,
	      "setting config directory", NULL) ;
      return FALSE ;
    }
    strcpy(confdir, path) ;
    strcpy(confdir + pathlen, CONFIG_DIRNAME) ;
  }

  if ( spooldir == NULL &&
       (spooldir = GetRegString(SERVICE_REGISTRY_KEY, SPOOL_REGNAME)) == NULL ) {
    if ( (spooldir = malloc(pathlen + strlen(SPOOL_DIRNAME) + 1)) == NULL ) {
      message(EVENTLOG_ERROR_TYPE, MSG_MEMORY,
	      "setting spool directory", NULL) ;
      return FALSE ;
    }
    strcpy(spooldir, path) ;
    strcpy(spooldir + pathlen, SPOOL_DIRNAME) ;
  }

  if ( postdir == NULL &&
       (postdir = GetRegString(SERVICE_REGISTRY_KEY, POST_REGNAME)) == NULL ) {
    if ( (postdir = malloc(pathlen + strlen(POST_DIRNAME) + 1)) == NULL ) {
      message(EVENTLOG_ERROR_TYPE, MSG_MEMORY,
	      "setting post directory", NULL) ;
      return FALSE ;
    }
    strcpy(postdir, path) ;
    strcpy(postdir + pathlen, POST_DIRNAME) ;
  }

  if ( inport == 0 ) {
    DWORD port = GetRegDWORD(SERVICE_REGISTRY_KEY, "ClientPort") ;

    if ( port )
      inport = htons((unsigned short)port) ;
  }

  if ( outport == 0 ) {
    DWORD port = GetRegDWORD(SERVICE_REGISTRY_KEY, "ServerPort") ;

    if ( port )
      outport = htons((unsigned short)port) ;
  }

  return TRUE ;
}

/* recursive pattern match with * wildchars */
int Match(char *pattern, char *string)
{
  char *star = strchr(pattern, '*') ;

  if ( star == NULL )	/* No globs, must match exactly */
    return strcmp(pattern, string) == 0 ;

  if ( star != pattern ) { /* match and remove stuff before star */
    if ( strncmp(pattern, string, star - pattern) != 0 )
      return FALSE ;
    string += star - pattern ;
  }

  while (*star == '*') star++ ;	/* find char after star */

  if ( *star != '\0' ) {  /* loop searching for thing after star */
    while ( (string = strchr(string, *star)) != NULL ) {
      if ( Match(star, string) )
	return TRUE ;
      string++ ;	/* skip over initial match char */
    }

    return FALSE ;
  }

  /* Nothing after star, so it must match */
  return TRUE ;
}

/*
 * $Log: common.c,v $
 * Revision 1.3  2000/01/14 21:22:34  angus
 * More changes to logging. Use new streams 1.31 ioctl to specify debugging info
 * for client & server connections. Make fmtout more secure against buffer
 * overruns, and add a new format specifier for binary strings with visible
 * control characters. Change logging interface to add a varargs log, and use
 * that for stream debugging. Modify debug flags, add more status returns.
 *
 * Revision 1.2  2000/01/14 03:40:04  angus
 * Many changes; added named pipe logging, sending output to client debug
 * process for in-situ service debugging. Use this mechanism for queue status,
 * and enable after-the-fact debugging by saving 100 logged messages. Add
 * symbollic names for debug flags, and Mutex debugging.
 *
 * Revision 1.1.1.1  1999/10/25 07:49:57  angus
 * Initial CVS import
 *
 */
