/* events.c $Id: events.c,v 1.1.1.1 1999/10/25 07:49:57 angus Exp $
 *
 * Copyright (C) Angus J. C. Duggan, 1997.
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Manage event table for NNTP buffer
 */

#include "nntp.h"
#include "messages.h"

static HANDLE *evTable = NULL ;
static int evTableSize = 0 ;
static int evTableSizeMax = 0 ;

HANDLE RegisterEvent(HANDLE hEvent)
{
  if ( hEvent != NULL ) {
    if ( !Lock(EVENT_MUTEX, INFINITE) ) /* Claim event table mutex */
      return NULL ;

    if ( evTableSize >= evTableSizeMax ) {
      evTableSizeMax = evTableSizeMax * 2 + 8 ;
      if ( (evTable = realloc(evTable, evTableSizeMax)) == NULL ) {
	message(EVENTLOG_ERROR_TYPE, MSG_MEMORY, "EventTableAdd", NULL) ;
	Unlock(EVENT_MUTEX) ;
	return NULL ;
      }
    }
    evTable[evTableSize++] = hEvent ;

    Unlock(EVENT_MUTEX) ; /* Release event table mutex */
  }

  return hEvent ;	/* return event handle */
}

HANDLE WithdrawEvent(HANDLE hEvent)
{
  int index ;
  HANDLE result = NULL ;

  if ( !Lock(EVENT_MUTEX, INFINITE) ) /* Claim event table mutex */
    return NULL ;

  for ( index = 0 ; index < evTableSize ; index++ )
    if ( evTable[index] == hEvent ) {
      evTable[index] = evTable[--evTableSize] ;
      result = hEvent ;
    }

  Unlock(EVENT_MUTEX) ; /* Release event table mutex */

  return result ;
}

HANDLE WaitForEvent(DWORD timeout)
{
  DWORD result ;
  HANDLE handle = INVALID_HANDLE_VALUE ;

  if ( !Lock(EVENT_MUTEX, INFINITE) )
    return INVALID_HANDLE_VALUE ;

  timeout *= 1000 ;	/* Convert to ms */

  /* Wait for all objects including Mutexes if all is TRUE, just non-mutex
     if all is not true */
  result = WaitForMultipleObjects(evTableSize, evTable, FALSE, timeout) ;

  if ( result >= WAIT_ABANDONED_0 && result < WAIT_ABANDONED_0 + evTableSize )
    handle = evTable[result - WAIT_ABANDONED_0] ;
  else if ( result >= WAIT_OBJECT_0 && result < WAIT_OBJECT_0 + evTableSize )
    handle = evTable[result - WAIT_OBJECT_0] ;
  else if ( result == WAIT_TIMEOUT )
    handle = NULL ;

  Unlock(EVENT_MUTEX) ;
    
  return handle ;
}

void DestroyEvents(void)
{
  int i ;

  /* This thread must own the mutex before destroying event table */
  if ( !Lock(EVENT_MUTEX, INFINITE) )
    return ;

  for ( i = 0 ; i < evTableSize ; i++ )
    CloseHandle(evTable[i]) ;

  if ( evTable != NULL )
    free(evTable) ;

  evTableSize = evTableSizeMax = 0 ;
  evTable = NULL ;

  Unlock(EVENT_MUTEX) ;
}

/*
 * $Log: events.c,v $
 * Revision 1.1.1.1  1999/10/25 07:49:57  angus
 * Initial CVS import
 *
 */
