/* groupdump.c $Id: groupdump.c,v 1.3 2000/01/14 21:22:34 angus Exp $
 *
 * Copyright (C) Angus J. C. Duggan, 1997-1998
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Group dump utility for NNTP Buffer.
 */

#include "nntp.h"
#include "messages.h"

char *program ;
int debuglevel = 0x80000000 ;

char *confdir = NULL ;
char *spooldir = NULL ;
char *postdir = NULL ;
unsigned short inport, outport ;

long remember_max = 0 ;

BOOL is_service = FALSE ;

char *pattern = NULL ;

static void usage(void)
{
  fprintf(stderr, "Usage: %s [-dconfigdir] [-sspooldir] [pattern]\n", program) ;
  fprintf(stderr,
	  "Copyright (C) Angus J. C. Duggan, 1997-1998. Freely redistributable software\n"
	  "See file LICENSE for details.\n") ;
  ExitProcess(0) ;
}

int parse_args(int argc, char *argv[])
{
  int pathlen ;
  char path[MAX_PATH], *arg ;

  pathlen = GetModuleFileName(NULL, path, MAX_PATH) ;
  if ( pathlen ) {
    char *slash = strrchr(path, '\\') ;

    if ( slash ) {
      pathlen = slash - path ;
      *slash = '\0' ;
    }
  }

  for ( program = *argv++ ; --argc ; argv++ ) {
    if (argv[0][0] == '-') {
      char ch = argv[0][1] ;

      if ( strchr("dsD", ch) != NULL ) {
	if ( argv[0][2] != '\0' )
	  arg = &argv[0][2] ;
	else if ( --argc )
	  arg = *++argv ;
	else
	  return 0 ;
      } else if ( argv[0][2] != '\0' )
        return 0 ;

      switch (ch) {
      case 'd':	/* set config directory */
        confdir = arg ;
	break;
      case 's':	/* set spool directory */
        spooldir = arg ;
	break;
      case 'D':	/* debug level service */
        debuglevel |= strtol(arg, &arg, 0) ;
	if ( debuglevel == 0 )
	  return 0 ;
        break ;
      default:
	return 0 ;
      }
    } else if ( pattern == NULL )
      pattern = *argv ;
    else
      return 0 ;
  }

  if ( program == NULL )
    return 0 ;

  if ( !DefaultDirectories(path, pathlen) )
    return 0 ;

  if ( pattern == NULL )
    pattern = "*" ;

  return 1 ;
}

void LogString(int type, char *str)
{
  fputs(str, stderr) ;
  fflush(stderr) ;
}

int DumpGroup(char *gname, void *passthrough)
{
  Group *group ;

  if ( !Match(pattern, gname) )
    return TRUE ;

  if ( !Lock(GROUP_MUTEX, INFINITE) )
    return FALSE ;

  printf("Group %s", gname) ;

  if ( (group = GroupDBGet(gname)) != NULL ) {
    switch (group->state) {
    case GROUP_ACTIVE:
      printf("  (ACTIVE)") ; break ;
    case GROUP_INACTIVE:
      printf("  (INACTIVE)") ; break ;
    case GROUP_BOGUS:
      printf("  (BOGUS)") ; break ;
    case GROUP_NEW:
      printf("  (NEW)") ; break ;
    default:
      printf("  (CORRUPT)") ; break ;
    }
    printf("  %d-%d (%d)", group->low, group->high, group->articles) ;
    printf("  Post %c", group->post) ;
    printf("  Mark %d\n", group->mark) ;
    printf("  Created %s", TimeAsString(&group->created)) ;
    printf("  Newnews %s\n", TimeAsString(&group->newnews)) ;
    printf("  Expired %s", TimeAsString(&group->expired)) ;
    printf("  Selected %s\n", TimeAsString(&group->selected)) ;
  } else
    printf("  Not found\n") ;
  
  Unlock(GROUP_MUTEX) ;

  return TRUE ;
}

int main(int argc, char *argv[])
{
  if ( !parse_args(argc, argv) )
    usage() ;

  if ( !InitGroupDB() ) {
    fprintf(stderr, "Can't initialise group database") ;
    return 0 ;
  }
    
  ForAllGroups(DumpGroup, NULL) ;

  CloseGroupDB() ;

  return 0 ;
}

/*
 * $Log: groupdump.c,v $
 * Revision 1.3  2000/01/14 21:22:34  angus
 * More changes to logging. Use new streams 1.31 ioctl to specify debugging info
 * for client & server connections. Make fmtout more secure against buffer
 * overruns, and add a new format specifier for binary strings with visible
 * control characters. Change logging interface to add a varargs log, and use
 * that for stream debugging. Modify debug flags, add more status returns.
 *
 * Revision 1.2  2000/01/14 03:40:04  angus
 * Many changes; added named pipe logging, sending output to client debug
 * process for in-situ service debugging. Use this mechanism for queue status,
 * and enable after-the-fact debugging by saving 100 logged messages. Add
 * symbollic names for debug flags, and Mutex debugging.
 *
 * Revision 1.1.1.1  1999/10/25 07:49:57  angus
 * Initial CVS import
 *
 */
