/* netlib.c $Id: netlib.c,v 1.3 2000/01/14 21:22:34 angus Exp $
 *
 * Copyright (C) Angus J. C. Duggan, 1997.
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Interface to streams library for NNTP buffer
 */

#include "nntp.h"
#include "messages.h"

/* When to show streams debugging */
static int cookie_level(long cookie)
{
  switch ( cookie ) {
  case SOCKSTREAM_DEBUG:
    return DEBUG_SOCKETS ;
  case FILESTREAM_DEBUG:
    return DEBUG_FILES ;
  case LINESTREAM_DEBUG_READ:
    return DEBUG_READLINE ;
  case LINESTREAM_DEBUG_WRITE:
    return DEBUG_WRITELINE ;
  case DEBUG_CLIENT:
  case DEBUG_CLIENT_DATA:
  case DEBUG_SERVER:
  case DEBUG_POSTING:
  case DEBUG_DOWNLOAD:
    return cookie ;
  }

  return -1 ; /* Always show */
}

/* How to show streams debugging */
static char *cookie_format(long cookie)
{
  switch ( cookie ) {
  case LINESTREAM_DEBUG_READ:
    return "Read: %S" ;
  case LINESTREAM_DEBUG_WRITE:
    return "Wrote: %S" ;
  case DEBUG_CLIENT:
  case DEBUG_CLIENT_DATA:
    return "CLIENT: %S" ;
  case DEBUG_SERVER:
  case DEBUG_DOWNLOAD:
    return "SERVER: %S" ;
  case DEBUG_POSTING:
    return "POST: %S" ;
  }

  return "%S" ;
}

void stream_debug(long cookie, char *format, ...)
{
  va_list args ;

  va_start(args, format) ;
  if ( (debuglevel & cookie_level(cookie)) != 0 ) {
    char buffer[1024] ;
    LogString(LOG_DEBUG, fmtout(buffer, 1024, format, args)) ;
  }
  va_end(args) ;
}

void stream_debugstr(long cookie, char *string, int length)
{
  debug(cookie_level(cookie), cookie_format(cookie), string, (long)length) ;
}

void stream_error(Stream *this, long err, va_list args)
{
  switch (err) {
  case SOCKSTREAM_ERROR_CLOSE: {
    SOCKET sock = va_arg(args, SOCKET) ;
    DWORD why = (DWORD)va_arg(args, long) ;

    message(EVENTLOG_ERROR_TYPE, MSG_CLOSESOCKET, ErrorText(why), NULL) ;
    break ;
  }
  case FILESTREAM_ERROR_CLOSE: {
    HANDLE sock = (HANDLE)va_arg(args, long) ;
    DWORD why = (DWORD)va_arg(args, long) ;

    message(EVENTLOG_ERROR_TYPE, MSG_CLOSEFILE, ErrorText(why), NULL) ;
    break ;
  }
  default: /* No such log message */
    break ;
  }
}

/*
 * $Log: netlib.c,v $
 * Revision 1.3  2000/01/14 21:22:34  angus
 * More changes to logging. Use new streams 1.31 ioctl to specify debugging info
 * for client & server connections. Make fmtout more secure against buffer
 * overruns, and add a new format specifier for binary strings with visible
 * control characters. Change logging interface to add a varargs log, and use
 * that for stream debugging. Modify debug flags, add more status returns.
 *
 * Revision 1.2  2000/01/14 03:40:04  angus
 * Many changes; added named pipe logging, sending output to client debug
 * process for in-situ service debugging. Use this mechanism for queue status,
 * and enable after-the-fact debugging by saving 100 logged messages. Add
 * symbollic names for debug flags, and Mutex debugging.
 *
 * Revision 1.1.1.1  1999/10/25 07:49:57  angus
 * Initial CVS import
 *
 */
