/* registry.c $Id: registry.c,v 1.1.1.1 1999/10/25 07:49:57 angus Exp $
 *
 * Copyright (C) Angus J. C. Duggan, 1997-1998.
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Registry handling functions for SMTP and NNTP service for Windows NT.
 */

#include "nntp.h"
#include "messages.h"

char *GetRegString(char *keyname, char *name)
{
  HKEY key ;
  char *result = NULL ;

  if ( RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyname,
		    0, KEY_READ, &key) == ERROR_SUCCESS ) {
    DWORD type, size = MAX_PATH ;
    char buffer[MAX_PATH], expansion[MAX_PATH], *value ;

    if ( RegQueryValueEx(key, name, NULL, &type, buffer,
			 &size) == ERROR_SUCCESS ) {
      if ( type == REG_SZ ) {
	value = buffer ;
      } else if ( type == REG_EXPAND_SZ ) {
	size = ExpandEnvironmentStrings(buffer, expansion, MAX_PATH) ;
	value = expansion ;
      } else
	size = 0 ;

      debug(DEBUG_REGISTRY, "Query %s\\%s type %lu size %lu value %s\n",
	    keyname, name, type, size, value) ;

      if ( size > 0 && (result = malloc(size + 1)) != NULL )
	strcpy(result, value) ;
    }

    if ( RegCloseKey(key) != ERROR_SUCCESS ) {
      message(EVENTLOG_ERROR_TYPE, MSG_REGCLOSE, ErrorText(GetLastError()),
	      NULL) ;
    }
  }

  return result ;
}

int SetRegString(char *keyname, char *name, char *value)
{
  HKEY key ;
  DWORD disp ;
  int result = TRUE ;

  if ( RegCreateKeyEx(HKEY_LOCAL_MACHINE, keyname,
		      0, NULL, 0, KEY_WRITE|KEY_READ, NULL, &key,
		      &disp) == ERROR_SUCCESS ) {
    if ( RegSetValueEx(key, name, 0, REG_EXPAND_SZ, value,
		       strlen(value)) != ERROR_SUCCESS ) {
      message(EVENTLOG_WARNING_TYPE, MSG_REGSET, keyname, name,
	      value, ErrorText(GetLastError()), NULL) ;
      result = FALSE ;
    }

    if ( RegCloseKey(key) != ERROR_SUCCESS )
      message(EVENTLOG_ERROR_TYPE, MSG_REGCLOSE, ErrorText(GetLastError()),
	      NULL) ;
  } else {
    message(EVENTLOG_WARNING_TYPE, MSG_REGCREATE, keyname,
	    ErrorText(GetLastError()), NULL) ;
    result = FALSE ;
  }

  return result ;
}

FILETIME *GetRegFiletime(char *keyname, char *name, LPFILETIME tptr)
{
  HKEY key ;
  FILETIME *result = NULL ;

  if ( RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyname,
		    0, KEY_READ, &key) == ERROR_SUCCESS ) {
    DWORD type, size = sizeof(FILETIME) ;
    char buffer[sizeof(FILETIME)] ;

    if ( RegQueryValueEx(key, name, NULL, &type, buffer,
			 &size) == ERROR_SUCCESS &&
	 type == REG_BINARY && size == sizeof(FILETIME) ) {
      debug(DEBUG_REGISTRY, "Query %s\\%s type %lu size %lu value FILETIME\n",
	    keyname, name, type, size) ;
      memcpy(tptr, buffer, size) ;
      result = tptr ;
    }

    if ( RegCloseKey(key) != ERROR_SUCCESS ) {
      message(EVENTLOG_ERROR_TYPE, MSG_REGCLOSE, ErrorText(GetLastError()),
	      NULL) ;
    }
  }

  return result ;
}

int SetRegFiletime(char *keyname, char *name, LPFILETIME tptr)
{
  HKEY key ;
  DWORD disp ;
  int result = TRUE ;

  if ( RegCreateKeyEx(HKEY_LOCAL_MACHINE, keyname,
		      0, NULL, 0, KEY_WRITE|KEY_READ, NULL, &key,
		      &disp) == ERROR_SUCCESS ) {
    if ( RegSetValueEx(key, name, 0, REG_BINARY, (char *)tptr,
		       sizeof(FILETIME)) != ERROR_SUCCESS ) {
      message(EVENTLOG_WARNING_TYPE, MSG_REGSET, keyname, name,
	      "time()", ErrorText(GetLastError()), NULL) ;
      result = FALSE ;
    }

    if ( RegCloseKey(key) != ERROR_SUCCESS )
      message(EVENTLOG_ERROR_TYPE, MSG_REGCLOSE, ErrorText(GetLastError()),
	      NULL) ;
  } else {
    message(EVENTLOG_WARNING_TYPE, MSG_REGCREATE, keyname,
	    ErrorText(GetLastError()), NULL) ;
    result = FALSE ;
  }

  return result ;
}

int SetRegDWORD(char *keyname, char *name, DWORD value)
{
  HKEY key ;
  DWORD disp ;
  int result = TRUE ;

  if ( RegCreateKeyEx(HKEY_LOCAL_MACHINE, keyname,
		      0, NULL, 0, KEY_WRITE|KEY_READ, NULL, &key,
		      &disp) == ERROR_SUCCESS ) {
    if ( RegSetValueEx(key, name, 0, REG_DWORD, (char *)&value,
		       sizeof(DWORD)) != ERROR_SUCCESS ) {
      char buffer[20] ;
      sprintf(buffer, "%d", value) ;
      message(EVENTLOG_WARNING_TYPE, MSG_REGSET, keyname, name,
	      buffer, ErrorText(GetLastError()), NULL) ;
      result = FALSE ;
    }

    if ( RegCloseKey(key) != ERROR_SUCCESS )
      message(EVENTLOG_ERROR_TYPE, MSG_REGCLOSE, ErrorText(GetLastError()),
	      NULL) ;
  } else {
    message(EVENTLOG_WARNING_TYPE, MSG_REGCREATE, keyname,
	    ErrorText(GetLastError()), NULL) ;
    result = FALSE ;
  }

  return result ;
}

DWORD GetRegDWORD(char *keyname, char *name)
{
  HKEY key ;
  DWORD result = 0 ;

  if ( RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyname,
		    0, KEY_READ, &key) == ERROR_SUCCESS ) {
    DWORD type, size = sizeof(DWORD) ;

    if ( RegQueryValueEx(key, name, NULL, &type, (char *)&result,
			 &size) != ERROR_SUCCESS ||
	 type != REG_DWORD || size != sizeof(DWORD) )
      result = 0 ;

    debug(DEBUG_REGISTRY, "Query %s\\%s type %lu size %lu value %lu\n",
	  keyname, name, type, size, result) ;

    if ( RegCloseKey(key) != ERROR_SUCCESS ) {
      message(EVENTLOG_ERROR_TYPE, MSG_REGCLOSE, ErrorText(GetLastError()),
	      NULL) ;
    }
  }

  return result ;
}

/*
 * $Log: registry.c,v $
 * Revision 1.1.1.1  1999/10/25 07:49:57  angus
 * Initial CVS import
 *
 */
