/* service.c $Id: service.c,v 1.2 2000/01/14 03:40:04 angus Exp $

  Parts blatently copied from Microsoft, and reformatted to my taste:

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Portions Copyright (C) 1993, 1994  Microsoft Corporation.
  All Rights Reserved.
  
  AUTHOR: Craig Link - Microsoft Developer Support

  Portions Copyright (C) 1997-1998 Angus J.C. Duggan
  */

#include <winsock2.h>
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <process.h>

#include "nntp.h"
#include "messages.h"

/* internal variables */
SERVICE_STATUS          ssStatus;       /* current status of the service */
SERVICE_STATUS_HANDLE   sshStatusHandle ;

BOOL is_service = FALSE ;

static BOOL ReportStatusToSCMgr(DWORD dwCurrentState,
				DWORD dwWin32ExitCode,
				DWORD dwWaitHint)
{
  static DWORD dwCheckPoint = 1;
  BOOL fResult = TRUE;

  if ( !debuglevel ) {	/* when debugging we don't report to the SCM */
    if (dwCurrentState == SERVICE_START_PENDING)
      ssStatus.dwControlsAccepted = 0;
    else
      ssStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP|SERVICE_ACCEPT_SHUTDOWN;

    ssStatus.dwCurrentState = dwCurrentState;
    ssStatus.dwWin32ExitCode = dwWin32ExitCode;
    ssStatus.dwWaitHint = dwWaitHint;

    if ( ( dwCurrentState == SERVICE_RUNNING ) ||
	 ( dwCurrentState == SERVICE_STOPPED ) )
      ssStatus.dwCheckPoint = 0;
    else
      ssStatus.dwCheckPoint = dwCheckPoint++;

    /* Report the status of the service to the service control manager. */
    if ( !(fResult = SetServiceStatus( sshStatusHandle, &ssStatus)) ) {
      message(EVENTLOG_ERROR_TYPE, MSG_STATUSFAIL,
	      ErrorText(GetLastError()), NULL) ;
    }
  }

  return fResult;
}

int InstallService(void)
{
  SC_HANDLE   schService;
  SC_HANDLE   schSCManager;
  TCHAR szPath[MAX_PATH];

  if ( !CreateDirectory(confdir, NULL) &&
       GetLastError() != ERROR_ALREADY_EXISTS ) {
    printf("Unable to install %s - create queue directory %s failed,\n %s\n",
	   SERVICE_DISPLAYNAME, confdir, ErrorText(GetLastError())) ;
    return FALSE ;
  }

  if ( GetModuleFileName(NULL, szPath, MAX_PATH) == 0 ) {
    printf("Unable to install %s - %s\n", SERVICE_DISPLAYNAME,
	   ErrorText(GetLastError())) ;
    return FALSE ;
  }

  if ( !SetRegString(SERVICE_REGISTRY_KEY, CONFIG_REGNAME, confdir) ||
       !SetRegString(SERVICE_REGISTRY_KEY, SPOOL_REGNAME, spooldir) ||
       !SetRegString(SERVICE_REGISTRY_KEY, POST_REGNAME, postdir) ||
       !SetRegString(EVENTLOG_REGISTRY_KEY, "EventMessageFile", szPath) ||
       !SetRegDWORD(EVENTLOG_REGISTRY_KEY, "TypesSupported",
		    EVENTLOG_ERROR_TYPE|EVENTLOG_WARNING_TYPE|EVENTLOG_INFORMATION_TYPE) ) {
    printf("Unable to install %s - can't create registry value, %s\n",
	   SERVICE_DISPLAYNAME, ErrorText(GetLastError())) ;
    return FALSE ;
  }
  
  schSCManager = OpenSCManager(NULL,              /* machine (NULL == local) */
			       NULL,           /* database (NULL == default) */
			       SC_MANAGER_ALL_ACCESS) ;   /* access required */
  if ( schSCManager ) {
    schService = CreateService(schSCManager,           /* SCManager database */
			       SERVICE_NAME,              /* name of service */
			       SERVICE_DISPLAYNAME,       /* name to display */
			       SERVICE_ALL_ACCESS,         /* desired access */
			       SERVICE_WIN32_OWN_PROCESS,    /* service type */
			       SERVICE_AUTO_START,             /* start type */
			       SERVICE_ERROR_NORMAL,   /* error control type */
			       szPath,                   /* service's binary */
			       NULL,               /* no load ordering group */
			       NULL,                    /* no tag identifier */
			       SERVICE_DEPENDENCIES,         /* dependencies */
			       NULL,                  /* LocalSystem account */
			       NULL);                         /* no password */

    if ( schService ) {
      printf("%s installed.\n", SERVICE_DISPLAYNAME );
      printf("Registry entries are in HKEY_LOCAL_MACHINE\\%s\n", SERVICE_REGISTRY_KEY) ;
      printf("  Config directory is %s\n", confdir) ;
      printf("  Spool directory is %s\n", spooldir) ;
      printf("  Posting directory is %s\n", postdir) ;
      CloseServiceHandle(schService);
    } else {
      printf("CreateService failed - %s\n", ErrorText(GetLastError()));
    }

    CloseServiceHandle(schSCManager);
  } else
    printf("OpenSCManager failed - %s\n", ErrorText(GetLastError()));

  return TRUE ;
}

int RemoveService(void)
{
  SC_HANDLE   schService;
  SC_HANDLE   schSCManager;
  int result = FALSE ; /* Assume failure unless success */

  schSCManager = OpenSCManager(NULL,              /* machine (NULL == local) */
			       NULL,           /* database (NULL == default) */
			       SC_MANAGER_ALL_ACCESS) ;   /* access required */
  if ( schSCManager ) {
    schService = OpenService(schSCManager, SERVICE_NAME, SERVICE_ALL_ACCESS);

    if (schService) {
      /* try to stop the service */
      if ( ControlService(schService, SERVICE_CONTROL_STOP, &ssStatus) ) {
	printf("Stopping %s.", SERVICE_DISPLAYNAME);
	Sleep( 1000 );

	while( QueryServiceStatus( schService, &ssStatus ) ) {
	  if ( ssStatus.dwCurrentState == SERVICE_STOP_PENDING ) {
	    printf(".");
	    Sleep( 1000 );
	  } else
	    break;
	}

	if ( ssStatus.dwCurrentState == SERVICE_STOPPED )
	  printf("\n%s stopped.\n", SERVICE_DISPLAYNAME );
	else
	  printf("\n%s failed to stop.\n", SERVICE_DISPLAYNAME);
      }

      /* now remove the service */
      if( DeleteService(schService) ) {
	printf("%s removed.\n", SERVICE_DISPLAYNAME );
	result = TRUE ;

	/* Make sure we are removing queue directory for service, not one
	   from command line */
	confdir = GetRegString(SERVICE_REGISTRY_KEY, CONFIG_REGNAME) ;

	/* Don't clear out directory first, only remove if empty */
	if ( confdir == NULL || !RemoveDirectory(confdir) ) {
	  printf("%s: Unable to remove config directory %s, %s\n",
		 SERVICE_DISPLAYNAME, confdir, ErrorText(GetLastError())) ;
	}

	spooldir = GetRegString(SERVICE_REGISTRY_KEY, SPOOL_REGNAME) ;

	/* Don't clear out directory first, only remove if empty */
	if ( spooldir == NULL || !RemoveDirectory(spooldir) ) {
	  printf("%s: Unable to remove spool directory %s, %s\n",
		 SERVICE_DISPLAYNAME, spooldir, ErrorText(GetLastError())) ;
	}

	postdir = GetRegString(SERVICE_REGISTRY_KEY, CONFIG_REGNAME) ;

	/* Don't clear out directory first, only remove if empty */
	if ( postdir == NULL || !RemoveDirectory(postdir) ) {
	  printf("%s: Unable to remove posting directory %s, %s\n",
		 SERVICE_DISPLAYNAME, postdir, ErrorText(GetLastError())) ;
	}

	/* Remove event log keys installed: other keys are created by
	   service addition automatically */
	if ( RegDeleteKey(HKEY_LOCAL_MACHINE, EVENTLOG_REGISTRY_KEY)
	     != ERROR_SUCCESS ) {
	  printf("%s: Unable to remove registry key HKEY_LOCAL_MACHINE\\%s, %s\n",
		 SERVICE_DISPLAYNAME, EVENTLOG_REGISTRY_KEY,
		 ErrorText(GetLastError())) ;
	}

      } else
	printf("DeleteService failed - %s\n", ErrorText(GetLastError()));

      CloseServiceHandle(schService);
    } else
      printf("OpenService failed - %s\n", ErrorText(GetLastError()));

    CloseServiceHandle(schSCManager);
  } else
    printf("OpenSCManager failed - %s\n", ErrorText(GetLastError()));

  return result ;
}

int UserService(int code)
{
  SC_HANDLE   schService;
  SC_HANDLE   schSCManager;
  int result = FALSE ; /* Assume failure unless success */

  schSCManager = OpenSCManager(NULL,              /* machine (NULL == local) */
			       NULL,           /* database (NULL == default) */
			       SC_MANAGER_CONNECT) ;   /* access required */
  if ( schSCManager ) {
    schService = OpenService(schSCManager, SERVICE_NAME, SERVICE_ALL_ACCESS);

    if (schService) { /* send message to service */
      if ( ControlService(schService, code, &ssStatus) )
        result = TRUE ;

      CloseServiceHandle(schService);
    } else
      printf("OpenService failed - %s\n", ErrorText(GetLastError()));

    CloseServiceHandle(schSCManager);
  } else
    printf("OpenSCManager failed - %s\n", ErrorText(GetLastError()));

  return result ;
}

int RestartService(void)
{
  SC_HANDLE   schService;
  SC_HANDLE   schSCManager;
  int result = FALSE ; /* Assume failure unless success */

  schSCManager = OpenSCManager(NULL,              /* machine (NULL == local) */
			       NULL,           /* database (NULL == default) */
			       SC_MANAGER_CONNECT) ;   /* access required */
  if ( schSCManager ) {
    schService = OpenService(schSCManager, SERVICE_NAME, SERVICE_ALL_ACCESS);

    if (schService) { /* send message to service */
      if ( StartService(schService, 0, NULL) )
        result = TRUE ;

      CloseServiceHandle(schService);
    } else
      printf("OpenService failed - %s\n", ErrorText(GetLastError()));

    CloseServiceHandle(schSCManager);
  } else
    printf("OpenSCManager failed - %s\n", ErrorText(GetLastError()));

  return result ;
}

/* Control events to signal when to stop service */
static HANDLE hClientConnected, hStopEvent ;
HANDLE hEvents[N_SIGNALS][ALL_CONTROLS] ;
int suspended[N_SIGNALS] ;

static VOID WINAPI service_ctrl(DWORD dwCtrlCode)
{
  switch( dwCtrlCode ) {
  case SERVICE_CONTROL_SHUTDOWN:/* Stop the service. */
  case SERVICE_CONTROL_STOP:    /* Stop the service. */
    ssStatus.dwCurrentState = SERVICE_STOP_PENDING;
    ssStatus.dwWaitHint = 5000 ;	/* 5 seconds to notice stop */
    if ( hStopEvent ) /* Signal to change state */
      SetEvent(hStopEvent);
    break;
  case SERVICE_CONTROL_INTERROGATE:    /* Update the service status. */
    break;
  case NNTP_CONTROL_START_POST:	/* Start posting immediately */
    suspended[SIGNAL_POST] = FALSE ;
    if ( hEvents[SIGNAL_POST][CONTROL_START] ) /* Signal to change state */
      SetEvent(hEvents[SIGNAL_POST][CONTROL_START]);
    break ;
  case NNTP_CONTROL_STOP_POST: /* Stop current and future posting */
    suspended[SIGNAL_POST] = TRUE ;
  case NNTP_CONTROL_ABORT_POST: /* Stop current posting, but allow future */
    if ( hEvents[SIGNAL_POST][CONTROL_STOP] ) /* Signal to change state */
      SetEvent(hEvents[SIGNAL_POST][CONTROL_STOP]);
    break ;
  case NNTP_CONTROL_START_SLURP: /* Start slurping immediately */
    suspended[SIGNAL_SLURP] = FALSE ;
    if ( hEvents[SIGNAL_SLURP][CONTROL_START] ) /* Signal to change state */
      SetEvent(hEvents[SIGNAL_SLURP][CONTROL_START]);
    break ;
  case NNTP_CONTROL_STOP_SLURP: /* Stop current and future slurping */
    suspended[SIGNAL_SLURP] = TRUE ;
  case NNTP_CONTROL_ABORT_SLURP: /* Stop current slurping, but allow future */
    if ( hEvents[SIGNAL_SLURP][CONTROL_STOP] ) /* Signal to change state */
      SetEvent(hEvents[SIGNAL_SLURP][CONTROL_STOP]);
    break ;
  case NNTP_CONTROL_START_EXPIRE: /* Start expiring immediately */
    suspended[SIGNAL_EXPIRE] = FALSE ;
    if ( hEvents[SIGNAL_EXPIRE][CONTROL_START] ) /* Signal to change state */
      SetEvent(hEvents[SIGNAL_EXPIRE][CONTROL_START]);
    break ;
  case NNTP_CONTROL_STOP_EXPIRE: /* Stop current and future expiring */
    suspended[SIGNAL_EXPIRE] = TRUE ;
  case NNTP_CONTROL_ABORT_EXPIRE: /* Stop current expiring, but allow future */
    if ( hEvents[SIGNAL_EXPIRE][CONTROL_STOP] ) /* Signal to change state */
      SetEvent(hEvents[SIGNAL_EXPIRE][CONTROL_STOP]);
    break ;
  default:    /* invalid control code */
    break;
  }

  ReportStatusToSCMgr(ssStatus.dwCurrentState, NO_ERROR, 0);
}

static int CheckServers(Server *server, void *passthrough)
{
  FILETIME *when = (FILETIME *)passthrough ;

  switch ( Lock(SERVER_MUTEX, 1) ) {
  case LockFailed:
  case LockTimeout:
    return FALSE ;	/* Somebody else is using servers, give up */
  }

  if ( server->connection != NULL &&
       CompareFileTime(&server->connecttime, when) < 0 )
    DisconnectServer(server) ;

  Unlock(SERVER_MUTEX) ;

  return TRUE ;
}

/* How do we determine our own address? Not as easy as it looks, in the
   presence of loopback (which may make the call come from localhost), and
   DHCP (which may give an external name which is different from the machine's
   own idea of the name), and multihoming, which may give the machine several
   addresses. The approach taken here is to check localhost (127.0.0.1)
   explicitly, or get our own hostname and call gethostbyname on that and
   check against the address list. */
int is_my_address(struct sockaddr_in *sin, int len)
{
  struct hostent *hent = NULL ;

  static struct in_addr localhost = { { 127, 0, 0, 1 } } ;
  static char hostname[MAX_HOSTLEN] = { '\0' } ;

  if ( len < sizeof(struct sockaddr_in) )
    return FALSE ;

  /* Special case for 127.0.0.1 (localhost) */
  if ( localhost.S_un.S_addr == sin->sin_addr.S_un.S_addr )
    return TRUE ;

  /* Get own host name */
  if ( hostname[0] == '\0' && gethostname(hostname, MAX_HOSTLEN) != 0 )
    return FALSE ;

  /* Get own host info */
  if ( (hent = gethostbyname(hostname)) != NULL &&
       hent->h_addrtype == AF_INET &&
       hent->h_length >= sizeof(struct in_addr)) {
    struct in_addr **addrs = (struct in_addr **)hent->h_addr_list ;

    /* Check all addresses */
    while ( *addrs ) {
      if ( (*addrs)->S_un.S_addr == sin->sin_addr.S_un.S_addr )
	return TRUE ;
      ++addrs ;
    }
  }

  return FALSE ;
}

VOID WINAPI service_main(DWORD argc, LPTSTR *argv)
{
  SOCKET listensock ;
  WSADATA wsaData;
  HANDLE hAcceptEvent, hPost, hSlurp, hExpire ;
  DWORD nClients = 0 ;
  DWORD threadid ;
  int index ;

  ThreadType(MAIN_THREAD) ;

  is_service = TRUE ;

  /* Set program, spooldir, portdir and confdir */
  if ( parse_args((int)argc, (char **)argv) == ACTION_USAGE ) {
    message(EVENTLOG_ERROR_TYPE, MSG_USAGE, SERVICE_DISPLAYNAME,
	    VERSION_STRING, SERVICE_NAME, NULL) ;
    return ;
  }

  if ( !debuglevel ) {
    /* register our service control handler: */
    sshStatusHandle = RegisterServiceCtrlHandler(SERVICE_NAME, service_ctrl) ;

    if ( !sshStatusHandle ) {
      message(EVENTLOG_ERROR_TYPE, MSG_HANDLERFAIL,
	      ErrorText(GetLastError()), NULL) ;
      goto cleanup ;
    }
  }

  /* SERVICE_STATUS members that don't change in example */
  ssStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS ;
  ssStatus.dwServiceSpecificExitCode = 0 ;

  /* report the status to the service control manager. */
  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* Start service; create events for control handler to signal, start
     loop */

  /* Register the stop event so the client can see it */
  hStopEvent = RegisterEvent(CreateEvent(NULL, TRUE, FALSE, NULL)) ;

  hClientConnected = CreateEvent(NULL, TRUE, FALSE, NULL) ;

  if ( hStopEvent == NULL || hClientConnected == NULL ) {
    message(EVENTLOG_ERROR_TYPE, MSG_CREATEEVENT,
	    ErrorText(GetLastError()), NULL) ;
    goto cleanup;
  }

  for ( index = 0 ; index < N_SIGNALS ; ++index ) {
    /* Unregistered auto-reset events for stop & start */
    hEvents[index][CONTROL_START] = CreateEvent(NULL, FALSE, FALSE, NULL) ;
    hEvents[index][CONTROL_STOP] = CreateEvent(NULL, FALSE, FALSE, NULL) ;

    if ( hEvents[index][CONTROL_START] == NULL ||
         hEvents[index][CONTROL_STOP] == NULL ) {
      message(EVENTLOG_ERROR_TYPE, MSG_CREATEEVENT,
              ErrorText(GetLastError()), NULL) ;
      goto cleanup;
    }

    hEvents[index][CONTROL_QUIT] = hStopEvent ;
    hEvents[index][CONTROL_CLIENT] = hClientConnected ;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* load newsfeeds file */
  if ( !InitNewsFeeds(NEWSFEEDS) )
    goto cleanup;

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* create the event object for listen socket association */
  hAcceptEvent = RegisterEvent(CreateEvent(NULL, TRUE, FALSE, NULL)) ;

  if ( hAcceptEvent == NULL) {
    message(EVENTLOG_ERROR_TYPE, MSG_CREATEEVENT,
	    ErrorText(GetLastError()), NULL) ;
    goto cleanup;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* Initialise group database and mutex */
  if ( !InitGroupDB())
    goto cleanup;

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* Initialise article database and mutex */
  if ( !InitArticleDB())
    goto cleanup;

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* Initialise message database and mutex */
  if ( !InitMessageDB())
    goto cleanup;

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* Initialise winsock */
  if ( WSAStartup(MAKEWORD(1,1), &wsaData) != 0) {
    message(EVENTLOG_ERROR_TYPE, MSG_WSASTARTUP,
	    ErrorText(WSAGetLastError()), NULL) ;
    goto cleanup ;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* Make sure winsock version is compatible. */
  if ( LOBYTE(wsaData.wVersion) != 1 || HIBYTE(wsaData.wVersion) != 1 ) {
    message(EVENTLOG_ERROR_TYPE, MSG_STACKVERSION, NULL) ;
    goto winsock_cleanup ;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto winsock_cleanup ;

  if ( INVALID_SOCKET == (listensock = socket(PF_INET, SOCK_STREAM, 0)) ) {
    message(EVENTLOG_ERROR_TYPE, MSG_LISTEN,
	    ErrorText(WSAGetLastError()), NULL) ;
    goto winsock_cleanup ;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto winsock_cleanup ;

  {
    struct servent *serviceentry = NULL ;
    struct sockaddr_in slisten ;	/* socket addresses */
    BOOL reuse = TRUE ;

    if ( (serviceentry = getservbyname("nntp", "tcp")) == NULL ) {
      message(EVENTLOG_ERROR_TYPE, MSG_NOTCPNNTP,
	      ErrorText(WSAGetLastError()), NULL) ;
      goto close_cleanup ;
    }

    if ( outport == 0 )
      outport = serviceentry->s_port ;

    if ( inport == 0 )
      inport = serviceentry->s_port ;

    /* Set up listener socket address */
    memset(&slisten, 0, sizeof(struct sockaddr_in)) ;
    slisten.sin_family = AF_INET ;
    slisten.sin_port = inport ;
    slisten.sin_addr.s_addr = INADDR_ANY ;

    if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
      goto close_cleanup ;

    if ( setsockopt(listensock, SOL_SOCKET, SO_REUSEADDR, (char *)&reuse,
		    sizeof(reuse)) != 0 ) {
      message(EVENTLOG_ERROR_TYPE, MSG_SETSOCKETOPT,
	      ErrorText(WSAGetLastError()), NULL) ;
    }

    if ( bind(listensock, (struct sockaddr *)&slisten, sizeof(struct sockaddr_in)) != 0 ) {
      message(EVENTLOG_ERROR_TYPE, MSG_BIND,
	      ErrorText(WSAGetLastError()), NULL) ;
      goto close_cleanup ;
    }
  }

  if ( WSAEventSelect(listensock, hAcceptEvent, FD_ACCEPT) == SOCKET_ERROR ) {
    message(EVENTLOG_ERROR_TYPE, MSG_EVENTSELECT,
	    ErrorText(GetLastError()), NULL) ;
    goto close_cleanup ;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto close_cleanup ;

  if ( SOCKET_ERROR == listen(listensock, 2) ) {
    message(EVENTLOG_ERROR_TYPE, MSG_LISTEN,
	    ErrorText(WSAGetLastError()), NULL) ;
    goto close_cleanup ;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto close_cleanup ;

  /* Create log thread */
  if ( CreateThread(NULL,	/* Security same as me */
                    0,		/* Same stack too */
                    LogThread, /* Entry point */
                    NULL, /* parameter */
                    0,		/* flags */
                    &threadid) == NULL ) {
    message(EVENTLOG_ERROR_TYPE, MSG_CREATETHREAD, "queue and debug",
            ErrorText(GetLastError()), NULL) ;
    goto close_cleanup ;
  }
	
  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto close_cleanup ;

  /* Get current server queue; POST, group WANT, article WANT */
  if ( (hPost = CreateThread(NULL,	/* Security same as me */
			     0,		/* Same stack too */
			     PostThread, /* Entry point */
			     NULL, /* parameter */
			     0,		/* flags */
			     &threadid)) == NULL ) {
    message(EVENTLOG_ERROR_TYPE, MSG_CREATETHREAD, "posting news",
	    ErrorText(GetLastError()), NULL) ;
    goto wait_cleanup ;
  }
	
  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto wait_cleanup ;

  if ( ( hSlurp = CreateThread(NULL,	/* Security same as me */
			      0,		/* Same stack too */
			      SlurpThread, /* Entry point */
			      NULL, /* parameter */
			      0,		/* flags */
			      &threadid)) == NULL ) {
    message(EVENTLOG_ERROR_TYPE, MSG_CREATETHREAD, "downloading news",
	    ErrorText(GetLastError()), NULL) ;
    goto wait_cleanup ;
  }
	
  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto wait_cleanup ;

  if ( (hExpire = CreateThread(NULL,	/* Security same as me */
			       0,		/* Same stack too */
			       ExpireThread, /* Entry point */
			       NULL, /* parameter */
			       0,		/* flags */
			       &threadid)) == NULL ) {
    message(EVENTLOG_ERROR_TYPE, MSG_CREATETHREAD, "expiring news",
	    ErrorText(GetLastError()), NULL) ;
    goto wait_cleanup ;
  }

  /* When running, do this: */
  if ( !ReportStatusToSCMgr(SERVICE_RUNNING, NO_ERROR, 0) )
    goto wait_cleanup ;

  {
    char strnum[20] ;
    if ( inport == outport ) {
      sprintf(strnum, "%d", (int)ntohs((unsigned short)inport)) ;
    } else {
      sprintf(strnum, "%d:%d", (int)ntohs((unsigned short)inport),
              (int)ntohs((unsigned short)outport)) ;
    }
    message(EVENTLOG_INFORMATION_TYPE, MSG_STARTED, SERVICE_DISPLAYNAME,
	    confdir, spooldir, postdir, strnum, NULL) ;
  }

  for (;;) {
    HANDLE hEvent ;
    DWORD timeout = GetRegDWORD(SERVICE_REGISTRY_KEY, DISCONNECT_TIMEOUT_NAME) ;
    if ( timeout <= 0 )
      timeout = DISCONNECT_TIMEOUT ;

    hEvent = WaitForEvent(timeout) ;

    if ( hEvent == hStopEvent || hEvent == INVALID_HANDLE_VALUE )
      break ;	/* quit when we're told to, or on error */

    if ( hEvent == hAcceptEvent ) { /* Accept connection */
      SOCKET clientsock ;
      DWORD threadid ;
      HANDLE hClient ;
      struct sockaddr_in sremote ;
      int remotelen  = sizeof(struct sockaddr_in) ;

      if ( (clientsock = accept(listensock, (struct sockaddr *)&sremote,
				&remotelen)) != INVALID_SOCKET ) {
	if ( !ResetEvent(hAcceptEvent) ) {
	  message(EVENTLOG_ERROR_TYPE, MSG_RESETEVENT,
		  ErrorText(WSAGetLastError()), NULL) ;
	}

	/* Security check; only accept connections from my machine */
	if ( !is_my_address(&sremote, remotelen) ) {
	  message(EVENTLOG_INFORMATION_TYPE, MSG_INSECURE,
		  inet_ntoa(sremote.sin_addr), NULL) ;
	  closesocket(clientsock) ;
	} else {
	  hClient = CreateThread(NULL,	/* Security same as me */
				 0,		/* Same stack too */
				 ClientThread, /* Entry point */
				 (LPVOID)&clientsock, /* parameter */
				 0,		/* flags */
				 &threadid) ;
	
	  if ( hClient == NULL ) {
	    message(EVENTLOG_ERROR_TYPE, MSG_CREATETHREAD, "NNTP Client",
		    ErrorText(GetLastError()), NULL) ;
	  } else {
	    /* Swap listener for client while serving client */
	    nClients++ ;
	    RegisterEvent(hClient) ;
	    SetEvent(hClientConnected) ;
	  }
	}
      } 
    } else if ( hEvent == NULL ) { /* timeout */
      FILETIME now ;

      /* Timeout is used to test when to disconnect servers */
      GetSystemTimeAsFileTime(&now) ;
      now = TimeAdd(&now, -(long)timeout) ;
      ForAllServers(CheckServers, &now, FALSE) ;
    } else { /* Client has terminated, can't be any other option */
      /* Remove client from table, add listener */
      WithdrawEvent(hEvent) ;
      CloseHandle(hEvent) ;
      nClients-- ;
      if ( nClients == 0 )
	ResetEvent(hClientConnected) ;
    }
  }

wait_cleanup:
  WithdrawEvent(hAcceptEvent) ;

  /* Wait on server threads finishing */
  while ( ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 3000) ) {
    if ( WaitForSingleObject(hPost, 2000) != WAIT_TIMEOUT )
      break ;
  }

  while ( ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 3000) ) {
    if ( WaitForSingleObject(hSlurp, 2000) != WAIT_TIMEOUT )
      break ;
  }

  while ( ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 3000) ) {
    if ( WaitForSingleObject(hExpire, 2000) != WAIT_TIMEOUT )
      break ;
  }

  /* Acquire posting, group and, article mutexes, so client threads can't be
     doing work */
  while ( ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 3000) ) {
    if ( Lock(POST_MUTEX, 2000) != LockTimeout )
      break ;
  }

  while ( ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 3000) ) {
    if ( Lock(GROUP_MUTEX, 2000) != LockTimeout )
      break ;
  }

  while ( ReportStatusToSCMgr(SERVICE_STOP_PENDING, NO_ERROR, 3000) ) {
    if ( Lock(ARTICLE_MUTEX, 2000) != LockTimeout )
      break ;
  }

close_cleanup:
  if ( closesocket(listensock) != 0 )
    message(EVENTLOG_ERROR_TYPE, MSG_CLOSESOCKET,
	    ErrorText(WSAGetLastError()), NULL) ;

winsock_cleanup:
  if ( WSACleanup() != 0 )
    message(EVENTLOG_ERROR_TYPE, MSG_WSACLEANUP,
	    ErrorText(WSAGetLastError()), NULL) ;

cleanup:
  /* Close any handles created */
  for ( index = 0 ; index < N_SIGNALS ; ++index ) {
    int controls ;

    for ( controls = 0 ; controls < ALL_CONTROLS ; ++controls )
      if ( hEvents[index][controls] != NULL )
        CloseHandle(hEvents[index][controls]) ;
  }

  CloseMessageDB() ;
  CloseArticleDB() ;
  CloseGroupDB() ;

  /* try to report the stopped status to the service control manager. */
  if ( sshStatusHandle )
    (VOID)ReportStatusToSCMgr(SERVICE_STOPPED, GetLastError(), 0);
  
  return ;
}

BOOL WINAPI ControlHandler(DWORD dwCtrlType)
{
  switch( dwCtrlType ) {
  case CTRL_BREAK_EVENT:  /* use Ctrl+C or Ctrl+Break to simulate */
  case CTRL_C_EVENT:      /* SERVICE_CONTROL_STOP in debug mode */
    printf("Stopping %s.\n", SERVICE_DISPLAYNAME);
    if ( hStopEvent )
      SetEvent(hStopEvent);

    return TRUE;
  }
  return FALSE;
}

/*
 * $Log: service.c,v $
 * Revision 1.2  2000/01/14 03:40:04  angus
 * Many changes; added named pipe logging, sending output to client debug
 * process for in-situ service debugging. Use this mechanism for queue status,
 * and enable after-the-fact debugging by saving 100 logged messages. Add
 * symbollic names for debug flags, and Mutex debugging.
 *
 * Revision 1.1.1.1  1999/10/25 07:49:57  angus
 * Initial CVS import
 *
 */
