/* timedate.c $Id: timedate.c,v 1.1.1.1 1999/10/25 07:49:57 angus Exp $
 *
 * Copyright (C) Angus J. C. Duggan, 1997-1998.
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Time and date functions for SMTP and NNTP service for Windows NT.
 */

#include "nntp.h"
#include "messages.h"

FILETIME TimeAdd(LPFILETIME when, long seconds)
{
  LONGLONG lsecs = (LONGLONG)seconds * 10000000 ; /* secs -> 100ns units */
  FILETIME ticks = *when ;

  *((LONGLONG *)&ticks) += lsecs ;

  return ticks ;
}

long TimeDiff(LPFILETIME from, LPFILETIME to)
{
  LONGLONG diff = *((LONGLONG *)to) - *((LONGLONG *)from) ;

  diff /= (LONGLONG)10000000 ;	/* 100ns units to seconds */

  return (long)diff ;
}

LPFILETIME DateTimeToFiletime(LPFILETIME when, char *nntpdate, char *nntptime)
{
  SYSTEMTIME stime ;
  WORD year = (nntpdate[0] - '0') * 10 + nntpdate[1] - '0' ;
  WORD month = (nntpdate[2] - '0') * 10 + nntpdate[3] - '0' ;
  WORD day = (nntpdate[4] - '0') * 10 + nntpdate[5] - '0' ;
  WORD hours = (nntptime[0] - '0') * 10 + nntptime[1] - '0' ;
  WORD mins = (nntptime[2] - '0') * 10 + nntptime[3] - '0' ;
  WORD secs = (nntptime[4] - '0') * 10 + nntptime[5] - '0' ;

  if ( year < 50 )
    year += 2000 ;
  else
    year += 1900 ;

  stime.wYear = year ;
  stime.wMonth = month ;	/* 1-based */
  stime.wDay = day ;		/* 1-based */
  stime.wHour = hours ;		/* 24-hour */
  stime.wMinute = mins ;
  stime.wSecond = secs ;
  stime.wMilliseconds = 0 ;

  if ( SystemTimeToFileTime(&stime, when) )
    return when ;

  return NULL ;
}

char *TimeAsString(LPFILETIME when)
{
  SYSTEMTIME stime ;
  static char string[15] ;

  if ( !FileTimeToSystemTime(when, &stime) )
    return NULL ;

  sprintf(string, "%02d%02d%02d %02d%02d%02d",
	  (stime.wYear % 100), stime.wMonth, stime.wDay,
	  stime.wHour, stime.wMinute, stime.wSecond) ;

  return string ;
}

/*
 * $Log: timedate.c,v $
 * Revision 1.1.1.1  1999/10/25 07:49:57  angus
 * Initial CVS import
 *
 */
