#ifndef _COMMON_H
#define _COMMON_H
/* Common.c
 *
 * Copyright (C) Angus J. C. Duggan, 1996
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Common functions for registry munging programs
 */

#define REGISTRY_SEPARATOR '\\'
#define REGISTRY_SEPARATOR_STRING "\\"

#define REG_KEY ((DWORD)-1)
#define REG_BOGUS ((DWORD)-2)

extern char *program ;

void *xalloc(void *ptr, size_t size) ;

void *extend(void *vector, DWORD *allocated, size_t size) ;

typedef enum { KV_None = 0,
	       KV_Add = 1,
	       KV_Delete = 2,
	       KV_Modify = 4,
	       KV_Set = 5 /* KV_Add|KV_Modify */ } KV_State ;

typedef struct _base Base ;
typedef struct _key Key ;
typedef struct _value Value ;

typedef union _keyvalue KeyValue ;

typedef struct _context {
  char *name ;	/* key name */
  HKEY hkey ;
  KeyValue *list ;
  struct _context *parent ;
} Context ;

typedef struct _iteration {
  int flags ;	/* flags iteration was opened with */
  Context *context ;
  FILE *fp ;
} Iteration ;

/* Find addresses for packed name and data */
#define KVNamePacked(kv) ((kv)->base.namelen < sizeof(char *))
#define KVName(kv) (KVNamePacked(kv) \
		    ? (char *)&((kv)->base.name) \
		    : (kv)->base.name)
#define KVDataPacked(kv) ((kv)->base.datalen <= sizeof(unsigned char *))
#define KVData(kv) (KVDataPacked(kv) \
		    ? (unsigned char *)&((kv)->base.data) \
		    : (kv)->base.data)

/* Common part for Key and Value classes */
struct _base {
  DWORD type ;
  char *name ;
  DWORD namelen ;
  unsigned char *data ;
  DWORD datalen ;
  Context *parent ;
  KeyValue *next ;
  KV_State state ;
} ;

struct _key {
  Base base ;	/* Must be first element in struct */
} ;

struct _value {
  Base base ;	/* Must be first element in struct */
} ;

union _keyvalue {
  Base base ;
  Key key;
  Value value ;
} ;

Iteration *IterateKey(char *keyname, REGSAM perms) ;
Iteration *IterateFile(char *keyname) ;
KeyValue *NextKeyVal(Iteration *it) ;
void EndIteration(Iteration *it) ;

void WriteKeyVal(FILE *outfile, KeyValue *this) ;
void FreeKeyVal(KeyValue *this) ;

HKEY ContextHKEY(Context *context, REGSAM perms) ;

char *KeyValName(KeyValue *kv, char *str, int *size) ;

#endif /* _COMMON_H */
