/* Regdump.c
 *
 * Copyright (C) Angus J. C. Duggan, 1996
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Dump registry keys in a way that can be used for regset and regdiff.
 */

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "common.h"

char *program ;

static void usage(void)
{
  fprintf(stderr, "Copyright (C) Angus J. C. Duggan, 1996-1997\n") ;
  fprintf(stderr, "Usage:\n  %s [key...]\n", program) ;
  fflush(stderr) ;
  exit(1) ;
}

int main (int argc, char *argv[])
{
  int verbose = 1 ;

  for (program = *argv++ ; --argc ; argv++) {
    if (argv[0][0] == '-') {
      switch (argv[0][1]) {
      case 'q':	/* quiet */
	verbose = 0;
	break;
      default:
	usage();
      }
    } else { /* dump key */
      Iteration *it = IterateKey(*argv, 0) ;

      if ( it ) {
	KeyValue *kv ;

	while ( (kv = NextKeyVal(it)) != NULL ) {
	  WriteKeyVal(stdout, kv) ;
	  FreeKeyVal(kv) ;
	}
	EndIteration(it) ;
      } else {
	fprintf(stderr, "Can't iterate over key %s\n", *argv) ;
      }
    }
  }

  return 0 ;
}
