/* netlib.c
 *
 * Copyright (C) Angus J. C. Duggan, 1997-1998.
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Interface to streams library for SMTP buffer
 */

#include "smtp.h"
#include "messages.h"

static int cookie_level(long cookie)
{
  switch ( cookie ) {
  case SOCKSTREAM_DEBUG:
    return DEBUG_SOCKETS ;
  case FILESTREAM_DEBUG:
    return DEBUG_FILES ;
  case LINESTREAM_DEBUG_READ:
    return DEBUG_READLINE ;
  case LINESTREAM_DEBUG_WRITE:
    return DEBUG_WRITELINE ;
  case SERVERSTREAM_DEBUG_READ:
    return DEBUG_FROMSERVER ;
  case QUEUEDSTREAM_DEBUG_WRITE:
    return DEBUG_TOCLIENT ;
  case QUEUEDSTREAM_DEBUG_READ:
  case CONNECTION_DEBUG_READ:
    return DEBUG_FROMCLIENT ;
  case SERVERSTREAM_DEBUG_WRITE:
  case CONNECTION_DEBUG_WRITE:
    return DEBUG_TOSERVER ;
  case SERVERSTREAM_DEBUG_CONNECT:
    return DEBUG_CONNECT ;
  case SMTPSTREAM_DEBUG:
    return DEBUG_SMTP ;
  }

  return -1 ; /* Always show */
}

void stream_debug(long cookie, char *format, ...)
{
  va_list args ;

  va_start(args, format) ;
  if ( (debuglevel & cookie_level(cookie)) != 0 ) {
    char buffer[1024] ;
    char *str = fmtout(buffer, 1024, format, args) ;
    fputs(str, stderr) ;
    fflush(stderr) ;
  }
  va_end(args) ;
}

void stream_debugstr(long cookie, char *string, int length)
{
  debugstr(cookie_level(cookie), string, length) ;
}

void stream_log(int code, long err)
{
  switch (code) {
  case STREAM_ERROR_CLOSE_SOCKET:
    message(EVENTLOG_ERROR_TYPE, MSG_CLOSESOCKET, ErrorText((DWORD)err), NULL) ;
    break ;
  case STREAM_ERROR_CLOSE_FILE:
    message(EVENTLOG_ERROR_TYPE, MSG_CLOSEFILE, ErrorText((DWORD)err), NULL) ;
    break ;
  default: /* No such log message */
    break ;
  }
}

/* end of netlib.c */
