/* queuestr.c
 *
 * (C) Angus J. C. Duggan, 1998
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Queued message stream for SMTP service for Windows NT.
 */

#include "smtp.h"
#include "messages.h"

#include "streami.h"

/* Queued message stream reads from underlying file. Writes are ignored. */
typedef struct {
  Stream *base ;
  enum { QUEUED_STREAM_COOKIE = 0x51554544 } cookie ; /* 'QUED' */
} QueuedStream ;

/* All writes trivially succeed */
static long queuedwriter(Stream *this, void *buffer, long bufsize)
{
#ifdef DEBUG
  stream_debug(QUEUEDSTREAM_DEBUG_WRITE, "[QW %04d] ", bufsize) ;
  stream_debugstr(QUEUEDSTREAM_DEBUG_WRITE, buffer, bufsize) ;
#endif

  return bufsize ;
}

/* Read from underlying file stream */
static long queuedreader(Stream *this, void *buffer, long bufsize)
{
  register QueuedStream *squeued = (QueuedStream *)this ;
  long rbytes ;

  /* Sanity check on stream */
  if ( squeued == NULL || squeued->cookie != QUEUED_STREAM_COOKIE ) {
    SetLastError(ERROR_INVALID_PARAMETER) ;
    return -1 ;
  }

  rbytes = ReadStream(UnderlyingStream(this), buffer, bufsize) ;

#ifdef DEBUG
  stream_debug(QUEUEDSTREAM_DEBUG_READ, "[QR %04d] ", rbytes) ;
  stream_debugstr(QUEUEDSTREAM_DEBUG_READ, buffer, rbytes) ;
#endif

  return rbytes ;
}

Stream *MakeQueuedStream(char *filename)
{
  Stream *files = NULL ;

  if ( (files = MakeFileStream(filename, FILESTREAM_READ)) != NULL ) {
    Stream *this ;
      
    if ( (this = NewStream(NULL, files, sizeof(QueuedStream),
                           queuedreader, queuedwriter, NULL, NULL)) != NULL ) {
      QueuedStream *squeued = (QueuedStream *)this ;

      squeued->cookie = QUEUED_STREAM_COOKIE ;

      return this ;
    }
  }

  while ( files )
    files = DestroyStream(files) ;

  return NULL ;
}

/* end of queuestr.c */
