/*
  service.c

  Parts blatently copied from Microsoft, and reformatted to my taste:

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Portions Copyright (C) 1993, 1994  Microsoft Corporation.
  All Rights Reserved.
  
  AUTHOR: Craig Link - Microsoft Developer Support

  Portions Copyright (C) 1997-1998 Angus J.C. Duggan
  */

#include "smtp.h"
#include "messages.h"

static int suspended ;

enum { CONTROL_QUIT, CONTROL_CLIENT, CONTROL_STOP, CONTROL_START,
       ALL_CONTROLS } ;

/* internal variables */
SERVICE_STATUS          ssStatus;       /* current status of the service */
SERVICE_STATUS_HANDLE   sshStatusHandle ;

static BOOL ReportStatusToSCMgr(DWORD dwCurrentState,
				DWORD dwWin32ExitCode,
				DWORD dwWaitHint)
{
  static DWORD dwCheckPoint = 1;
  BOOL fResult = TRUE;

  if ( !debuglevel ) {	/* when debugging we don't report to the SCM */
    if (dwCurrentState == SERVICE_START_PENDING)
      ssStatus.dwControlsAccepted = 0;
    else
      ssStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP|SERVICE_ACCEPT_SHUTDOWN;

    ssStatus.dwCurrentState = dwCurrentState;
    ssStatus.dwWin32ExitCode = dwWin32ExitCode;
    ssStatus.dwWaitHint = dwWaitHint;

    if ( ( dwCurrentState == SERVICE_RUNNING ) ||
	 ( dwCurrentState == SERVICE_STOPPED ) )
      ssStatus.dwCheckPoint = 0;
    else
      ssStatus.dwCheckPoint = dwCheckPoint++;

    /* Report the status of the service to the service control manager. */
    if ( !(fResult = SetServiceStatus( sshStatusHandle, &ssStatus)) ) {
      message(EVENTLOG_ERROR_TYPE, MSG_STATUSFAIL,
	      ErrorText(GetLastError()), NULL) ;
    }
  }

  return fResult;
}

int InstallService(void)
{
  SC_HANDLE   schService;
  SC_HANDLE   schSCManager;
  TCHAR szPath[MAX_PATH];

  if ( !CreateDirectory(queuedir, NULL) &&
       GetLastError() != ERROR_ALREADY_EXISTS ) {
    printf("Unable to install %s - create queue directory %s failed,\n %s\n",
	   SERVICE_DISPLAYNAME, queuedir, ErrorText(GetLastError())) ;
    return FALSE ;
  }

  if ( GetModuleFileName( NULL, szPath, MAX_PATH ) == 0 ) {
    printf("Unable to install %s - %s\n", SERVICE_DISPLAYNAME,
	   ErrorText(GetLastError())) ;
    return FALSE ;
  }

  if ( !SetRegString(SERVICE_REGISTRY_KEY, "MAILHOST", mailhost) ||
       !SetRegString(SERVICE_REGISTRY_KEY, "QUEUEDIR", queuedir) ||
       !SetRegString(SERVICE_REGISTRY_KEY, "POSTMASTER", postmaster) ||
       !SetRegString(EVENTLOG_REGISTRY_KEY, "EventMessageFile", szPath) ||
       !SetRegDWORD(EVENTLOG_REGISTRY_KEY, "TypesSupported",
		    EVENTLOG_ERROR_TYPE|EVENTLOG_WARNING_TYPE|EVENTLOG_INFORMATION_TYPE) ) {
    printf("Unable to install %s - can't create registry value, %s\n",
	   SERVICE_DISPLAYNAME, ErrorText(GetLastError())) ;
    return FALSE ;
  }
  
  schSCManager = OpenSCManager(NULL,              /* machine (NULL == local) */
			       NULL,           /* database (NULL == default) */
			       SC_MANAGER_ALL_ACCESS) ;   /* access required */
  if ( schSCManager ) {
    schService = CreateService(schSCManager,           /* SCManager database */
			       SERVICE_NAME,              /* name of service */
			       SERVICE_DISPLAYNAME,       /* name to display */
			       SERVICE_ALL_ACCESS,         /* desired access */
			       SERVICE_WIN32_OWN_PROCESS,    /* service type */
			       SERVICE_AUTO_START,             /* start type */
			       SERVICE_ERROR_NORMAL,   /* error control type */
			       szPath,                   /* service's binary */
			       NULL,               /* no load ordering group */
			       NULL,                    /* no tag identifier */
			       SERVICE_DEPENDENCIES,         /* dependencies */
			       NULL,                  /* LocalSystem account */
			       NULL);                         /* no password */

    if ( schService ) {
      printf("%s installed.\n", SERVICE_DISPLAYNAME );
      printf("Registry entries are in HKEY_LOCAL_MACHINE\\%s\n", SERVICE_REGISTRY_KEY) ;
      printf("  MAILHOST is %s\n", mailhost) ;
      printf("  QUEUEDIR is %s\n", queuedir) ;
      printf("  POSTMASTER is %s\n", postmaster) ;
      CloseServiceHandle(schService);
    } else {
      printf("CreateService failed - %s\n", ErrorText(GetLastError()));
    }

    CloseServiceHandle(schSCManager);
  } else
    printf("OpenSCManager failed - %s\n", ErrorText(GetLastError()));

  return TRUE ;
}

int RemoveService(void)
{
  SC_HANDLE   schService;
  SC_HANDLE   schSCManager;
  int result = FALSE ; /* Assume failure unless success */

  schSCManager = OpenSCManager(NULL,              /* machine (NULL == local) */
			       NULL,           /* database (NULL == default) */
			       SC_MANAGER_ALL_ACCESS) ;   /* access required */
  if ( schSCManager ) {
    schService = OpenService(schSCManager, SERVICE_NAME, SERVICE_ALL_ACCESS);

    if (schService) {
      /* try to stop the service */
      if ( ControlService(schService, SERVICE_CONTROL_STOP, &ssStatus) ) {
	printf("Stopping %s.", SERVICE_DISPLAYNAME);
	Sleep( 1000 );

	while( QueryServiceStatus( schService, &ssStatus ) ) {
	  if ( ssStatus.dwCurrentState == SERVICE_STOP_PENDING ) {
	    printf(".");
	    Sleep( 1000 );
	  } else
	    break;
	}

	if ( ssStatus.dwCurrentState == SERVICE_STOPPED )
	  printf("\n%s stopped.\n", SERVICE_DISPLAYNAME );
	else
	  printf("\n%s failed to stop.\n", SERVICE_DISPLAYNAME);
      }

      /* Make sure we are removing queue directory for service, not one
	 from command line */
      queuedir = GetRegString(SERVICE_REGISTRY_KEY, "QUEUEDIR") ;

      /* now remove the service */
      if( DeleteService(schService) ) {
	printf("%s removed.\n", SERVICE_DISPLAYNAME );
	result = TRUE ;

	/* Should clear out directory first */
	if ( !RemoveDirectory(queuedir) ) {
	  printf("%s: Unable to remove queue directory %s, %s\n",
		 SERVICE_DISPLAYNAME, queuedir, ErrorText(GetLastError())) ;
	}

	/* Remove event log keys installed: other keys are created by
	   service addition automatically */
	if ( RegDeleteKey(HKEY_LOCAL_MACHINE, EVENTLOG_REGISTRY_KEY)
	     != ERROR_SUCCESS ) {
	  printf("%s: Unable to remove registry key HKEY_LOCAL_MACHINE\\%s, %s\n",
		 SERVICE_DISPLAYNAME, EVENTLOG_REGISTRY_KEY,
		 ErrorText(GetLastError())) ;
	}

      } else
	printf("DeleteService failed - %s\n", ErrorText(GetLastError()));

      CloseServiceHandle(schService);
    } else
      printf("OpenService failed - %s\n", ErrorText(GetLastError()));

    CloseServiceHandle(schSCManager);
  } else
    printf("OpenSCManager failed - %s\n", ErrorText(GetLastError()));

  return result ;
}

int UserService(int code)
{
  SC_HANDLE   schService;
  SC_HANDLE   schSCManager;
  int result = FALSE ; /* Assume failure unless success */

  schSCManager = OpenSCManager(NULL,              /* machine (NULL == local) */
			       NULL,           /* database (NULL == default) */
			       SC_MANAGER_CONNECT) ;   /* access required */
  if ( schSCManager ) {
    schService = OpenService(schSCManager, SERVICE_NAME, SERVICE_ALL_ACCESS);

    if (schService) { /* send message to service */
      if ( ControlService(schService, code, &ssStatus) )
        result = TRUE ;

      CloseServiceHandle(schService);
    } else
      printf("OpenService failed - %s\n", ErrorText(GetLastError()));

    CloseServiceHandle(schSCManager);
  } else
    printf("OpenSCManager failed - %s\n", ErrorText(GetLastError()));

  return result ;
}

int RestartService(void)
{
  SC_HANDLE   schService;
  SC_HANDLE   schSCManager;
  int result = FALSE ; /* Assume failure unless success */

  schSCManager = OpenSCManager(NULL,              /* machine (NULL == local) */
			       NULL,           /* database (NULL == default) */
			       SC_MANAGER_CONNECT) ;   /* access required */
  if ( schSCManager ) {
    schService = OpenService(schSCManager, SERVICE_NAME, SERVICE_ALL_ACCESS);

    if (schService) { /* send message to service */
      if ( StartService(schService, 0, NULL) )
        result = TRUE ;

      CloseServiceHandle(schService);
    } else
      printf("OpenService failed - %s\n", ErrorText(GetLastError()));

    CloseServiceHandle(schSCManager);
  } else
    printf("OpenSCManager failed - %s\n", ErrorText(GetLastError()));

  return result ;
}

static HANDLE hEvents[ALL_CONTROLS] ;

static VOID WINAPI service_ctrl(DWORD dwCtrlCode)
{
  /* Handle the requested control code. */

  switch( dwCtrlCode ) {
  case SERVICE_CONTROL_STOP:    /* Stop the service. */
  case SERVICE_CONTROL_SHUTDOWN:    /* Stop the service. */
    ssStatus.dwCurrentState = SERVICE_STOP_PENDING;
    if ( hEvents[CONTROL_QUIT] )
      SetEvent(hEvents[CONTROL_QUIT]);
    break;
  case SERVICE_CONTROL_INTERROGATE:    /* Update the service status. */
    break;
  case SMTP_CONTROL_START_MAIL:
    suspended = FALSE ;
    if ( hEvents[CONTROL_START] )
      SetEvent(hEvents[CONTROL_START]) ;
    break ;
  case SMTP_CONTROL_STOP_MAIL:
    suspended = TRUE ;
  case SMTP_CONTROL_ABORT_MAIL:
    if ( hEvents[CONTROL_STOP] )
      SetEvent(hEvents[CONTROL_STOP]) ;
    break ;
  default:    /* invalid control code */
    break;
  }

  ReportStatusToSCMgr(ssStatus.dwCurrentState, NO_ERROR, 0);
}

/* How do we determine our own address? Not as easy as it looks, in the
   presence of loopback (which may make the call come from localhost), and
   DHCP (which may give an external name which is different from the machine's
   own idea of the name), and multihoming, which may give the machine several
   addresses. The approach taken here is to check localhost (127.0.0.1)
   explicitly, or get our own hostname and call gethostbyname on that and
   check against the address list. */
int is_my_address(struct sockaddr_in *sin, int len)
{
  struct hostent *hent = NULL ;

  static struct in_addr localhost = { { 127, 0, 0, 1 } } ;
  static char hostname[MAX_HOSTLEN] = { '\0' } ;

  if ( len < sizeof(struct sockaddr_in) )
    return FALSE ;

  /* Special case for 127.0.0.1 (localhost) */
  if ( localhost.S_un.S_addr == sin->sin_addr.S_un.S_addr )
    return TRUE ;

  /* Get own host name */
  if ( hostname[0] == '\0' && gethostname(hostname, MAX_HOSTLEN) != 0 )
    return FALSE ;

  /* Get own host info */
  if ( (hent = gethostbyname(hostname)) != NULL &&
       hent->h_addrtype == AF_INET &&
       hent->h_length >= sizeof(struct in_addr)) {
    struct in_addr **addrs = (struct in_addr **)hent->h_addr_list ;

    /* Check all addresses */
    while ( *addrs ) {
      if ( (*addrs)->S_un.S_addr == sin->sin_addr.S_un.S_addr )
	return TRUE ;
      ++addrs ;
    }
  }

  return FALSE ;
}

static int lastfile = 0 ;

static int LastFile(char *filename, void *data)
{
  int *lastptr = data ;
  char *tail = strrchr(filename, '\\') ;
  int num = atoi(tail + 1) ;

  if ( num > *lastptr )
    *lastptr = num ;

  return 1 ;
}

/* Make new name for queueing files */
static char *queuefilename(char *buffer, long bufsize)
{
  sprintf(buffer, "%s\\%d" Q_OK_EXTN, queuedir, ++lastfile) ;

  return buffer ;
}

VOID WINAPI service_main(DWORD argc, LPTSTR *argv)
{
  SOCKET listensock ;
  WSADATA wsaData;
  DWORD ready, delay, disconnect ;
  Stream *conns = NULL ;
  FileQueue *queue = NULL ;

  /* Set program, mailhost, postmaster and queuedir */
  if ( parse_args((int)argc, (char **)argv) == ACTION_USAGE ) {
    message(EVENTLOG_ERROR_TYPE, MSG_USAGE, SERVICE_DISPLAYNAME,
	    VERSION_STRING, SERVICE_NAME, NULL) ;
    return ;
  }

  if ( !debuglevel ) {
    /* register our service control handler: */
    sshStatusHandle = RegisterServiceCtrlHandler(SERVICE_NAME, service_ctrl) ;

    if ( !sshStatusHandle ) {
      message(EVENTLOG_ERROR_TYPE, MSG_HANDLERFAIL,
	      ErrorText(GetLastError()), NULL) ;
      goto cleanup ;
    }
  }

  /* SERVICE_STATUS members that don't change in example */
  ssStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS ;
  ssStatus.dwServiceSpecificExitCode = 0 ;

  /* report the status to the service control manager. */
  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* Start service; create events for control handler to signal, start
     loop */

  /* Manual reset for quit and socket events */
  hEvents[CONTROL_QUIT] = CreateEvent(NULL, TRUE, FALSE, NULL);
  hEvents[CONTROL_CLIENT] = CreateEvent(NULL, TRUE, FALSE, NULL);

  /* Auto-reset events for stop and start */
  hEvents[CONTROL_START] = CreateEvent(NULL, FALSE, FALSE, NULL);
  hEvents[CONTROL_STOP] = CreateEvent(NULL, FALSE, FALSE, NULL);

  if ( hEvents[CONTROL_QUIT] == NULL || hEvents[CONTROL_CLIENT] == NULL ||
       hEvents[CONTROL_START] == NULL || hEvents[CONTROL_STOP] == NULL ) {
    message(EVENTLOG_ERROR_TYPE, MSG_CREATEEVENT,
	    ErrorText(GetLastError()), NULL) ;
    goto cleanup;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* Initialise winsock */
  if ( WSAStartup(MAKEWORD(1,1), &wsaData) != 0) {
    message(EVENTLOG_ERROR_TYPE, MSG_WSASTARTUP,
	    ErrorText(WSAGetLastError()), NULL) ;
    goto cleanup ;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto cleanup ;

  /* Make sure winsock version is compatible. */
  if ( LOBYTE(wsaData.wVersion) != 1 || HIBYTE(wsaData.wVersion) != 1 ) {
    message(EVENTLOG_ERROR_TYPE, MSG_STACKVERSION, NULL) ;
    goto winsock_cleanup ;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto winsock_cleanup ;

  if ( INVALID_SOCKET == (listensock = socket(PF_INET, SOCK_STREAM, 0)) ) {
    message(EVENTLOG_ERROR_TYPE, MSG_LISTEN,
	    ErrorText(WSAGetLastError()), NULL) ;
    goto winsock_cleanup ;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto winsock_cleanup ;

  {
    struct servent *serviceentry = NULL ;
    struct sockaddr_in slisten ;	/* socket addresses */
    BOOL reuse = TRUE ;

    if ( (serviceentry = getservbyname("smtp", "tcp")) == NULL ) {
      message(EVENTLOG_ERROR_TYPE, MSG_NOTCPSMTP,
	      ErrorText(WSAGetLastError()), NULL) ;
      goto close_cleanup ;
    }

    if ( outport == 0 )
      outport = serviceentry->s_port ;

    if ( inport == 0 )
      inport = serviceentry->s_port ;

    /* Set up listener socket address */
    memset(&slisten, 0, sizeof(struct sockaddr_in)) ;
    slisten.sin_family = AF_INET ;
    slisten.sin_port = inport ;
    slisten.sin_addr.s_addr = INADDR_ANY ;

    if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
      goto close_cleanup ;

    if ( setsockopt(listensock, SOL_SOCKET, SO_REUSEADDR, (char *)&reuse,
		    sizeof(reuse)) != 0 ) {
      message(EVENTLOG_ERROR_TYPE, MSG_SETSOCKETOPT,
	      ErrorText(WSAGetLastError()), NULL) ;
    }

    if ( bind(listensock, (struct sockaddr *)&slisten, sizeof(struct sockaddr_in)) != 0 ) {
      message(EVENTLOG_ERROR_TYPE, MSG_BIND,
	      ErrorText(WSAGetLastError()), NULL) ;
      goto close_cleanup ;
    }
  }

  if ( WSAEventSelect(listensock, hEvents[CONTROL_CLIENT], FD_ACCEPT) == SOCKET_ERROR ) {
    message(EVENTLOG_ERROR_TYPE, MSG_EVENTSELECT,
	    ErrorText(GetLastError()), NULL) ;
    goto close_cleanup ;
  }

  if ( !ReportStatusToSCMgr(SERVICE_START_PENDING, NO_ERROR, 3000) )
    goto close_cleanup ;

  if ( SOCKET_ERROR == listen(listensock, 2) ) {
    message(EVENTLOG_ERROR_TYPE, MSG_LISTEN,
	    ErrorText(WSAGetLastError()), NULL) ;
    goto close_cleanup ;
  }

  /* When running, do this: */
  if ( !ReportStatusToSCMgr(SERVICE_RUNNING, NO_ERROR, 0) )
    goto close_cleanup ;

  {
    char strnum[20] ;
    if ( inport == outport ) {
      sprintf(strnum, "%d", (int)ntohs((unsigned short)inport)) ;
    } else {
      sprintf(strnum, "%d:%d", (int)ntohs((unsigned short)inport),
              (int)ntohs((unsigned short)outport)) ;
    }
    message(EVENTLOG_INFORMATION_TYPE, MSG_STARTED, SERVICE_DISPLAYNAME,
            mailhost, postmaster, queuedir, strnum, NULL) ;
  }

  ready = GetRegDWORD(SERVICE_REGISTRY_KEY, READY_TIMEOUT_NAME) ;
  if ( ready <= 0 ) 	/* Default if silly value */
    ready = READY_TIMEOUT ;

  disconnect = GetRegDWORD(SERVICE_REGISTRY_KEY, DISCONNECT_TIMEOUT_NAME) ;
  if ( disconnect <= 0 ) 	/* Default if silly value */
    disconnect = DISCONNECT_TIMEOUT ;

  delay = GetRegDWORD(SERVICE_REGISTRY_KEY, READY_WAIT_NAME) ;
  if ( delay <= 0 ) 	/* Default if silly value */
    delay = READY_WAIT_DEFAULT ;

  (void)ForAllFiles(queuedir, Q_ALL_EXTN, LastFile, &lastfile) ;

  for ( ;; delay = ready ) {
    FILETIME now, then ;
    int disconnected = 0 ;

    if ( conns && queue ) /* Don't delay if we have stuff in the queue */
      delay = 0 ;

    do {

      if ( conns && delay > disconnect ) /* Shorter time until we disconnect */
        delay = disconnect ;

      GetSystemTimeAsFileTime(&now) ;
      then = TimeAdd(&now, delay) ;

      if ( suspended ) {
        SetRegString(SERVICE_REGISTRY_KEY, SMTP_STATUS, "Mail sending suspended") ;
      } else if ( FileTimeToLocalFileTime(&then, &now) ) {
        char buffer[80] ;
        SetRegString(SERVICE_REGISTRY_KEY, SMTP_STATUS,
                     formatstr(buffer, 80, "Next send is at %t", &now)) ;
      }

      if ( debuglevel != 0 )
        delay /= 10 ;

      switch ( WaitForMultipleObjects(ALL_CONTROLS, hEvents, FALSE, delay * 1000) ) {
      case WAIT_TIMEOUT:
        if ( delay && conns ) { /* Timed out waiting for more stuff */
          do {
            conns = DestroyStream(conns) ;
          } while ( conns ) ;
          disconnected = 1 ; /* Don't send after disconnection */
        } /* FALLTHRU */
      case WAIT_OBJECT_0 + CONTROL_START: /* Look at queue immediately */
        delay = 0 ;
        break ;
      case WAIT_OBJECT_0 + CONTROL_QUIT: /* Stop service */
        goto smtp_quit ;
      case WAIT_OBJECT_0 + CONTROL_CLIENT: { /* Connection pending */
        SOCKET insock ;
        struct sockaddr_in sremote ;
        int remotelen = sizeof(struct sockaddr_in) ;

        if ( (insock = accept(listensock, (struct sockaddr *)&sremote,
                              &remotelen)) != INVALID_SOCKET ) {
          unsigned long nonblock = 0 ;

          if ( !ResetEvent(hEvents[CONTROL_CLIENT]) ) {
            message(EVENTLOG_ERROR_TYPE, MSG_RESETEVENT,
                    ErrorText(WSAGetLastError()), NULL) ;
          }

          if ( WSAEventSelect(insock, NULL, 0) == SOCKET_ERROR ) {
            message(EVENTLOG_ERROR_TYPE, MSG_CLEAREVENT,
                    ErrorText(WSAGetLastError()), NULL) ;
          }

          if ( ioctlsocket(insock, FIONBIO, &nonblock) == SOCKET_ERROR ) {
            message(EVENTLOG_ERROR_TYPE, MSG_NONBLOCK,
                    ErrorText(WSAGetLastError()), NULL) ;
          }

          /* Security check: only accept connections from same machine */
          if ( !is_my_address(&sremote, remotelen) ) {
            message(EVENTLOG_INFORMATION_TYPE, MSG_INSECURE,
                    inet_ntoa(sremote.sin_addr), NULL) ;
            closesocket(insock) ;
          } else { /* Connection OK, let's see what they want... */
            Stream *client ;

            if ( (client = MakeSocketStream(insock)) != NULL ) {
              if ( conns ) {
                int status = IoctlStream(conns, CONNECTION_CHANGE,
                                         client, queuefilename) ;
                /* Do what with status? */
              } else
                conns = MakeConnectionStream(suspended ? NULL : mailhost,
                                             client, queuefilename) ;

              if ( conns ) {
                char buffer[BUFSIZ] ;
                long rbytes ;
                int status ;

                while ( (rbytes = ReadStream(conns, buffer, BUFSIZ)) > 0 ) {
                  if ( !WriteStreamAll(conns, buffer, rbytes) )
                    break ;
                }

                /* Remove client from connection */
                status = IoctlStream(conns, CONNECTION_CHANGE, NULL, NULL) ;

                if ( (status & STATUS_ESERVER) != 0 ) {
                  do {
                    conns = DestroyStream(conns) ;
                  } while ( conns ) ;
                } else
                  delay = 0 ; /* Have at it */
              }

              while ( client ) /* Close client connection */
                client = DestroyStream(client) ;
            } else {
              closesocket(insock) ;
              message(EVENTLOG_ERROR_TYPE, MSG_CREATESTREAM,
                      ErrorText(GetLastError()), NULL) ;
            }
          }
        }
        break ;
      }
      case WAIT_OBJECT_0 + CONTROL_STOP:
        while ( conns )
          conns = DestroyStream(conns) ;
        delay = ready ;
        break ;
      default:
        GetSystemTimeAsFileTime(&now) ;
        delay = TimeDiff(&now, &then) ;
        break ;
      }
    } while ( delay > 0 ) ;

    if ( suspended || disconnected )
      continue ;

    /* Check if we've buffered any messages */
    if ( queue == NULL )
      (void)ForAllFiles(queuedir, Q_OK_EXTN, ReadQueue, &queue) ;

    /* If we have messages queued, try to output them */
    if ( queue ) {
      FileQueue *next = queue->next ;
      Stream *queues = NULL ;
      int result = STATUS_ESERVER ; /* By default, didn't finish */

      /* Open file stream onto queue file */
      if ( (queues = MakeQueuedStream(queue->name)) != NULL ) {
        if ( conns ) {
          if ( (IoctlStream(conns, CONNECTION_CHANGE, queues, NULL) & STATUS_ESERVER) != 0 ) {
            do {
              conns = DestroyStream(conns) ;
            } while ( conns ) ;
          }
        } else
          conns = MakeConnectionStream(mailhost, queues, NULL) ;

        if ( conns ) {
          char buffer[BUFSIZ] ;
          long rbytes ;
          
          while ( (rbytes = ReadStream(conns, buffer, BUFSIZ)) > 0 ) {
            if ( !WriteStreamAll(conns, buffer, rbytes) )
              break ;
          }

          /* Remove queue client from connection, find out what happened */
          result = IoctlStream(conns, CONNECTION_CHANGE, NULL, NULL) ;

          if ( (result & (STATUS_ESERVER|STATUS_EMESSAGE|STATUS_EAGAIN)) != 0 ) {
            do {
              conns = DestroyStream(conns) ;
            } while ( conns ) ;
          }

          /* Reconnect if message error and more to do */
          if ( (result & (STATUS_EMESSAGE|STATUS_EAGAIN)) != 0 &&
               next != NULL )
            conns = MakeConnectionStream(mailhost, NULL, NULL) ;
        }

        while ( queues )
          queues = DestroyStream(queues) ;
      } else
        result |= STATUS_ECLIENT ; /* Something wrong, can't make stream */

      if ( (result & STATUS_ECLIENT) != 0 ) {
	/* Fatal error; problems reading message from disc */
	BounceMessage(queue->name, BOUNCE_FATAL, NULL) ;
      } else if ( (result & STATUS_EMESSAGE) != 0 ) {
	/* Message on disc incorrect, return to sender */
	BounceMessage(queue->name, BOUNCE_RETURN, queuefilename) ;
      } else if ( (result & STATUS_EAGAIN) != 0 ) {
	/* Resend message, it wasn't my fault it was delayed */
	BounceMessage(queue->name, BOUNCE_RESEND, NULL) ;
      } else if ( (result & STATUS_ESERVER) == 0 ) {
	char summary[MAX_SUMMARY] ;
	/* Sent message successfully */
	message(EVENTLOG_INFORMATION_TYPE, MSG_SENT,
		Summary(queue->name, summary, MAX_SUMMARY),
		mailhost, NULL) ; /* log message */
	if ( !DeleteFile(queue->name) ) {
	  message(EVENTLOG_ERROR_TYPE, MSG_DELETEFILE, queue->name,
		  ErrorText(GetLastError()), NULL) ;
	}
      }

      free(queue) ;
      queue = next ;
    }
  }

smtp_quit:
  SetRegString(SERVICE_REGISTRY_KEY, SMTP_STATUS, "SMTP cache terminated") ;

close_cleanup:
  if ( closesocket(listensock) != 0 )
    message(EVENTLOG_ERROR_TYPE, MSG_CLOSESOCKET,
	    ErrorText(WSAGetLastError()), NULL) ;

winsock_cleanup:
  if ( WSACleanup() != 0 )
    message(EVENTLOG_ERROR_TYPE, MSG_WSACLEANUP,
	    ErrorText(WSAGetLastError()), NULL) ;

cleanup:
  /* Close any handles created */
  {
    int index ;

    for ( index = 0 ; index < ALL_CONTROLS ; ++index )
      if ( hEvents[index] != NULL )
        CloseHandle(hEvents[index]) ;
  }

  /* try to report the stopped status to the service control manager. */
  if ( sshStatusHandle )
    (VOID)ReportStatusToSCMgr(SERVICE_STOPPED, GetLastError(), 0);
  
  return ;
}

BOOL WINAPI ControlHandler(DWORD dwCtrlType)
{
  switch( dwCtrlType ) {
  case CTRL_BREAK_EVENT:  /* use Ctrl+C or Ctrl+Break to simulate */
  case CTRL_C_EVENT:      /* SERVICE_CONTROL_STOP in debug mode */
    printf("Stopping %s.\n", SERVICE_DISPLAYNAME);
    if ( hEvents[CONTROL_QUIT] )
      SetEvent(hEvents[CONTROL_QUIT]);

    return TRUE;
  }
  return FALSE;
}

/* end of service.c */
