#ifndef SMTP_H
#define SMTP_H

/* smtp.h
 *
 * Copyright (C) Angus J. C. Duggan, 1997-1998
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Common definitions for SMTP buffer service for Windows NT
 *
 */

#include <winsock2.h>
#include <windows.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <assert.h>
#include <malloc.h>
#include <process.h>
#include <string.h>
#include <assert.h>

/* Include my streams library */
#include <streams.h>

#include "smtpstr.h"
#include "servstr.h"
#include "queuestr.h"
#include "connects.h"

/***** generally-accessible variables *****/
extern char *program, *mailhost, *queuedir, *postmaster ;
extern unsigned short inport, outport ;
extern int debuglevel ;

#define SERVICE_APPNAME "SMTP"
#define SERVICE_NAME "SMTPBuffer"
#define SERVICE_DISPLAYNAME "SMTP Buffer Service"
#define SERVICE_DEPENDENCIES "EventLog\0\0" /* "dep1\0\dep2\0\0" */
#define SERVICE_REGISTRY_KEY ("System\\CurrentControlSet\\Services\\" SERVICE_NAME)
#define EVENTLOG_REGISTRY_KEY ("System\\CurrentControlSet\\Services\\EventLog\\Application\\" SERVICE_NAME)

#define VERSION_STRING "1.10"

#define Q_BASENAME "\\smtpq"
#define Q_ALL_EXTN ".*"
#define Q_OK_EXTN ".out"
#define Q_ERROR_EXTN ".err"

#define CONNECT_TIMEOUT_NAME "ConnectTimeout"
#define CONNECT_TIMEOUT 5 /* seconds to wait for connection */
#define READY_TIMEOUT_NAME "SendInterval"
#define READY_TIMEOUT (10 * 60) /* Timeout between server checks */
#define READY_WAIT_NAME "SendWait"
#define READY_WAIT_DEFAULT (60 * 5) /* 5 minutes wait before first send */
#define DISCONNECT_TIMEOUT_NAME "DisconnectTimeout"
#define DISCONNECT_TIMEOUT (60) /* Seconds to wait for disconnection */

#define SMTP_STATUS "Status"

#define MAX_SUMMARY 512	/* Max length for summary in messages */
#define MAX_HOSTLEN 256 /* guess */

enum { DEBUG_FILES = 0x1, DEBUG_SOCKETS = 0x2,
       DEBUG_READLINE = 0x4, DEBUG_WRITELINE = 0x8,
       DEBUG_QUEUE = 0x10, DEBUG_MATCH = 0x20,
       DEBUG_SUMMARY = 0x40, DEBUG_SMTP = 0x80,
       DEBUG_RESPONSE = 0x100, DEBUG_BOUNCE = 0x200,
       DEBUG_CONNECT = 0x400, DEBUG_REGISTRY = 0x800,
       DEBUG_FROMCLIENT = 0x1000, DEBUG_TOCLIENT = 0x2000,
       DEBUG_FROMSERVER = 0x4000, DEBUG_TOSERVER = 0x8000
} ;

/***** Functions defined in service.c *****/
int InstallService(void) ;
int RemoveService(void) ;
int UserService(int code) ;
int RestartService(void) ;

/* Private Service control codes */
enum { SMTP_CONTROL_START_MAIL = 128,
       SMTP_CONTROL_ABORT_MAIL,
       SMTP_CONTROL_STOP_MAIL
} ;

VOID WINAPI service_main(DWORD dwArgc, LPTSTR *lpszArgv)  ;

BOOL WINAPI ControlHandler(DWORD dwCtrlType) ;

/***** Functions defined in common.c *****/

char *fmtout(char *buffer, int bufsize, char *format, va_list args) ;
void debug(int level, char *format, ...) ;
void debugstr(int level, char *string, long len) ;
char *formatstr(char *buffer, int bufsize, char *format, ...) ;
void message(WORD type, DWORD id, ...) ;

/* Type is
   EVENTLOG_ERROR_TYPE	Error event
   EVENTLOG_WARNING_TYPE	Warning event
   EVENTLOG_INFORMATION_TYPE	Information event
   */

typedef struct FileQueue {
  struct FileQueue *next ;
  char name[MAX_PATH] ;
} FileQueue ;

int ForAllFiles(char *directory, char *extn,
                int (*fn)(char *filename, void *passthrough),
                void *passthrough) ;
int ReadQueue(char *filename, void *data) ;

char *ErrorText(DWORD err) ;

char *Summary(char *filename, char *summary, long sumsize) ;

int Match(char *pattern, char *string) ;

int DefaultDirectories(void) ;

/***** Functions defined in registry.c *****/

char *GetRegString(char *keyname, char *name) ;
int SetRegString(char *keyname, char *name, char *value) ;
FILETIME *GetRegFiletime(char *keyname, char *name, FILETIME *tptr) ;
int SetRegFiletime(char *keyname, char *name, FILETIME *tptr) ;
DWORD GetRegDWORD(char *keyname, char *name) ;
int SetRegDWORD(char *keyname, char *name, DWORD value) ;

/***** Functions defined in timedate.c *****/

FILETIME TimeAdd(LPFILETIME when, long seconds) ;
long TimeDiff(LPFILETIME from, LPFILETIME to) ;
LPFILETIME DateTimeToFiletime(LPFILETIME when, char *nntpdate, char *nntptime) ;
char *TimeAsString(LPFILETIME when) ;

/***** Functions defined in smtp.c *****/

enum { BOUNCE_FATAL, BOUNCE_RETURN, BOUNCE_RESEND } ;

void BounceMessage(char *filename, int how, QueueNameFn *getname) ;

enum { ACTION_INSTALL, ACTION_UNINSTALL, ACTION_DEBUG, ACTION_DONE,
       ACTION_START, ACTION_CONTROL, ACTION_USAGE } ;

int parse_args(int argc, char *argv[]) ;

#endif /* SMTP_H */
