/* headstr.c
 *
 * Copyright (C) Angus J. C. Duggan, 1997
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Message header streams
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <memory.h>
#include <string.h>
#include <assert.h>

#include "streams.h"
#include "streami.h"

/*****************************************************************************/

/* Head stream is for reading messages headers. It uses a line-buffered
   underlying streams to read lines up to <CRLF>. */
static long headreader(Stream *this, void *buffer, long bufsize)
{
  long nbytes ;
  char *line ;

  /* Must be callee buffered */
  if ( this == NULL || buffer == NULL || bufsize != 0 ) {
    SetLastError(ERROR_INVALID_PARAMETER) ;
    return -1 ;
  }

  nbytes = ReadStream(UnderlyingStream(this), &line, 0) ;

  if ( nbytes <= 0 )	/* Error in underlying stream */
    return nbytes ;
  
  if ( nbytes == 2 && line[0] == '\r' && line[1] == '\n' )
    return 0 ;	/* Found <CRLF> */

  *((char **)buffer) = line ;

  return nbytes ;
}

Stream *MakeHeadStream(Stream *under)
{
  Stream *this = NewStream(NULL, under, 0, headreader, NULL, NULL, NULL) ;

  return this ;
}

/* End of headstr.c */
