#ifndef AJCD_STREAMS_H
#define AJCD_STREAMS_H

/* streams.h $Id: streams.h,v 1.2 2000/01/14 21:20:04 angus Exp $
 *
 * Copyright (C) Angus J. C. Duggan, 1997-2000
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Streams support for network and filter programs
 */

#define STREAMS_VERSION 131 /* 1.31 */

typedef struct _Stream Stream ;

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN /* Don't include most definitions */
#include <winsock2.h>
typedef SOCKET StreamSocket ;
#else
typedef socket StreamSocket ;
#endif

#include <stdio.h>
#include <stdarg.h>

/***** Imported functions *****/

#ifndef NDEBUG
extern void stream_debugstr(long cookie, char *string, int length) ;
extern void stream_debug(long cookie, char *format, ...) ;
#endif

/***** Base class functions defined in streams.c *****/

typedef long Handler(Stream *this, void *buffer, long bufsize) ;
typedef long Ioctl(Stream *this, long op, va_list args) ;
typedef void Error(Stream *this, long err, va_list args) ;
typedef void Destructor(Stream *this) ;

Stream *UnderlyingStream(Stream *this) ;
Stream *NewStream(Stream *this, Stream *under, short extrabytes,
		  Handler reader, Handler writer, Ioctl ioctl,
                  Error error, Destructor destructor) ;
Stream *DestroyStream(Stream *this) ;
long ReadStream(Stream *this, void *buffer, long bufsize) ;
long WriteStream(Stream *this, void *buffer, long bufsize) ;
long WriteStreamAll(Stream *this, void *buffer, long bufsize) ;
long IoctlStream(Stream *this, long op, ...) ;
void ErrorStream(Stream *this, long err, ...) ;

void LockStream(Stream *this) ;
void UnlockStream(Stream *this) ;

long SendStream(Stream *this, char *format, ...) ;

/* Generic Ioctls, supported by IoctlStream */
enum { STREAM_CLEAREOF = 0x43454f46, /* 'CEOF' */
       STREAM_CLEARERR = 0x43455252, /* 'CERR' */
       STREAM_REWIND = 0x52574e44, /* 'RWND' */
       STREAM_POSITION = 0x504f534e, /* 'POSN' */
       STREAM_DEBUG = 0x44425547, /* 'DBUG' */
} ;

/* Socket streams */
enum { SOCKSTREAM_DEBUG = 0x524f434b /* 'SOCK' */ } ; /* Debug cookies */
enum { SOCKSTREAM_ERROR_CLOSE = 0x43534f4b /* 'CSOK' */ } ; /* Errors */
Stream *MakeSocketStream(StreamSocket socket, Error socketerror) ;

/* File streams */
enum { FILESTREAM_READ, FILESTREAM_WRITE, FILESTREAM_READWRITE } ; 
enum { FILESTREAM_DEBUG = 0x46494c45 /* 'FILE' */ } ; /* Debug cookies */
enum { FILESTREAM_ERROR_CLOSE = 0x4346494c /* 'CFIL' */ } ; /* Errors */
Stream *MakeFileStream(char *filename, int access, Error fileerror) ;

/* Standard I/O streams */
enum { STDIOSTREAM_DEBUG = 0x5354494f /* 'STIO' */ } ; /* Debug cookies */
Stream *MakeStdioStream(FILE *fp) ;

/* Line streams */
enum { LINESTREAM_TERM = 0x4c494e54, /* 'LINT' */
       LINESTREAM_AGAIN = 0x4c494e41 /* 'LINA' */ } ; /* LineStream Ioctls */
enum { LINESTREAM_DEBUG_READ = 0x4c494e52 /* 'LINR' */,
       LINESTREAM_DEBUG_WRITE = 0x4c494e57 /* 'LINW' */ } ; /* Debug cookies */
Stream *MakeLineStream(Stream *under, char *term) ;
long Response(Stream *this) ;
long Reason(Stream *this, char *buffer, int bufsize) ;

/* Dot-terminated streams */
Stream *MakeDotStream(Stream *under) ;

/* Header streams */
Stream *MakeHeadStream(Stream *under) ;

/* Body streams */
Stream *MakeBodyStream(Stream *under) ;

/* Search streams */
enum { SEARCHSTREAM_RESULT = 0x53434852 , /* 'SCHR' */
       SEARCHSTREAM_AGAIN = 0x53434841 /* 'SCHA' */ } ; /* Search Ioctls */
Stream *MakeSearchStream(Stream *under, void *search, int len) ;

/*
 * $Log: streams.h,v $
 * Revision 1.2  2000/01/14 21:20:04  angus
 * Add STREAM_DEBUG ioctl, to modify returned debugging information. Update
 * release number to 1.31.
 *
 * Revision 1.1.1.1  1999/10/24 07:06:53  angus
 * Initial CVS import
 *
 */
#endif /* end of streams.h */
