#ifndef AJCD_STREAMS_H
#define AJCD_STREAMS_H

/* streams.h
 *
 * Copyright (C) Angus J. C. Duggan, 1997-1998
 *
 * Freely redistributable software. See file LICENSE for details.
 *
 * Streams support for network and filter programs
 */

#define STREAMS_VERSION 120 /* 1.20 */

typedef struct _Stream Stream ;

#ifdef WIN32
#include <windows.h>
typedef SOCKET StreamSocket ;
#else
typedef socket StreamSocket ;
#endif

/***** Imported functions *****/

#ifdef DEBUG
extern void stream_debugstr(long cookie, char *string, int length) ;
extern void stream_debug(long cookie, char *format, ...) ;
#endif

extern void stream_log(int code, long err) ;

/***** Base class functions defined in streams.c *****/

typedef long Handler(Stream *this, void *buffer, long bufsize) ;
typedef long Ioctl(Stream *this, int op, va_list args) ;
typedef void Destructor(Stream *this) ;

Stream *UnderlyingStream(Stream *this) ;
Stream *NewStream(Stream *this, Stream *under, short extrabytes,
		  Handler reader, Handler writer, Ioctl ioctl,
		  Destructor destructor) ;
Stream *DestroyStream(Stream *this) ;
long ReadStream(Stream *this, void *buffer, long bufsize) ;
long WriteStream(Stream *this, void *buffer, long bufsize) ;
long WriteStreamAll(Stream *this, void *buffer, long bufsize) ;
long IoctlStream(Stream *this, int op, ...) ;

void LockStream(Stream *this) ;
void UnlockStream(Stream *this) ;

long SendStream(Stream *this, char *format, ...) ;

/* Generic Ioctls, supported by IoctlStream */
enum { STREAM_CLEAREOF = 0x43454f46, /* 'CEOF' */
       STREAM_CLEARERR = 0x43455252, /* 'CERR' */
       STREAM_REWIND = 0x52574e44, /* 'RWND' */
       STREAM_POSITION = 0x504f534e /* 'POSN' */ } ;

/* Socket streams */
enum { SOCKSTREAM_DEBUG = 0x524f434b /* 'SOCK' */ } ; /* Debug cookies */
enum { STREAM_ERROR_CLOSE_SOCKET = 0x43534f4b /* 'CSOK' */ } ; /* Errors */
Stream *MakeSocketStream(StreamSocket socket) ;

/* File streams */
enum { FILESTREAM_READ, FILESTREAM_WRITE, FILESTREAM_READWRITE } ; 
enum { FILESTREAM_DEBUG = 0x46494c45 /* 'FILE' */ } ; /* Debug cookies */
enum { STREAM_ERROR_CLOSE_FILE = 0x4346494c /* 'CFIL' */ } ; /* Errors */
Stream *MakeFileStream(char *filename, int access) ;

/* Standard I/O streams */
enum { STDIOSTREAM_DEBUG = 0x5354494f /* 'STIO' */ } ; /* Debug cookies */
Stream *MakeStdioStream(FILE *fp) ;

/* Line streams */
enum { LINESTREAM_TERM, LINESTREAM_AGAIN } ; /* LineStream Ioctls */
enum { LINESTREAM_DEBUG_READ = 0x4c494e52 /* 'LINR' */,
       LINESTREAM_DEBUG_WRITE = 0x4c494e57 /* 'LINW' */ } ; /* Debug cookies */
Stream *MakeLineStream(Stream *under, char *term) ;
long Response(Stream *this) ;
long Reason(Stream *this, char *buffer, int bufsize) ;

/* Dot-terminated streams */
Stream *MakeDotStream(Stream *under) ;

/* Header streams */
Stream *MakeHeadStream(Stream *under) ;

/* Body streams */
Stream *MakeBodyStream(Stream *under) ;

/* Search streams */
enum { SEARCHSTREAM_RESULT, SEARCHSTREAM_AGAIN } ;
Stream *MakeSearchStream(Stream *under, void *search, int len) ;

#endif /* end of streams.h */
